#ifndef BEATFINDER_H
#define BEATFINDER_H

#include <vector>
#include <string>
#include <cmath>
#include "samplemath.h"
#include "functor.h"

//enum beatFinderTypes { BEATFINDER_STRAIGHTLINE , BEATFINDER_LINEARREGRESSION };

template<class T>
class BeatFinder
{
public:
    static bool isBeat ( std::basic_string<T> *slice, size_t from, size_t to, size_t lookaheadTime = 9, short decibelDifference = 10 );
    //lookaheadTime = 9 & decibelDifference = 10 >> amen
};

template<class T>
bool BeatFinder<T>::isBeat ( std::basic_string<T> *slice, size_t from, size_t loopLength, size_t lookaheadTime, short decibelDifference )
{
    /*
     * http://en.wikipedia.org/wiki/Decibel
     * L_db = 20 * log10 ( A1 / A0 );
     * audition: L_db = 10 (sensitivity), rise time = 9ms (lookahead)
     * 44100 samples / s
     * 9 ms = 0.009s ~ 44100 * 0.009 samples = 396.9 samples
    */

    double amplitudea0, amplitudea1, amplitudeb0, amplitudeb1,
    lookaheadSamples, lookaheadSamplesPlusEpsilon,
    lookbehindSamples, lookbehindSamplesPlusEpsilon,
    strictness, epsilon, tmp;
    bool aa0IsSet, aa1IsSet, ab0IsSet, ab1IsSet;
    size_t i, j, k, l;

    if ( loopLength - from < 2 )
        return false;

    /*
    if ( lookaheadTime * 0.001 * 44100 > SHRT_MAX )
    {
    	fprintf ( stderr, "B\n");
    	throw;
    }*/

    tmp = 0.001 * (double) lookaheadTime * 2 * 44100;
    lookaheadSamples = floor ( tmp );
    lookbehindSamples = lookaheadSamples;
    strictness = 0.0075;
    epsilon = strictness * lookaheadSamples;
    lookaheadSamplesPlusEpsilon = floor ( tmp + epsilon );
    lookbehindSamplesPlusEpsilon = lookaheadSamplesPlusEpsilon;

    if ( loopLength - from < lookaheadSamples )
        return false;

    aa0IsSet = false;
    i = from + 2;
    for ( ; i < loopLength - 2; i += 2 )
    {
        if (
            !aa0IsSet &&
            ( SampleMath::isPeak ( fabs( slice->at ( i - 2 ) ), fabs ( slice->at ( i ) ), fabs ( slice->at ( i + 2 ) ) ) )
        )
        {
            amplitudea0 = slice->at ( i );
            aa0IsSet = true;
            j = i;
            i += 2;
            break;
        }
    }

    aa1IsSet = false;
    for ( ; i < loopLength - 2; i += 2 )
    {
        if (
            ( ( i - j ) > lookaheadSamples ) &&
            ( ( i - j ) <= lookaheadSamplesPlusEpsilon ) &&
            !aa1IsSet &&
            ( SampleMath::isPeak ( fabs( slice->at ( i - 2 ) ), fabs ( slice->at ( i ) ), fabs ( slice->at ( i + 2 ) ) ) )
        )
        {
            amplitudea1 = slice->at ( i );
            aa1IsSet = true;
            break;
        }
    }

    ab0IsSet = false;
    if ( k > 1 )
        k = from - 2;
    for ( ; k > 1; k -= 2 )
    {
        if (
            !ab0IsSet &&
            ( SampleMath::isPeak ( fabs( slice->at ( k - 2 ) ), fabs ( slice->at ( k ) ), fabs ( slice->at ( k + 2 ) ) ) )
        )
        {
            amplitudeb0 = slice->at ( k );
            ab0IsSet = true;
            l = k;
            if ( k > 1 )
                k -= 2;
            break;
        }
    }

    ab1IsSet = false;
    for ( ; k > 1; k -= 2 )
    {
        if (
            ( ( l - k ) > lookbehindSamples ) &&
            ( ( l - k ) <= lookbehindSamplesPlusEpsilon )&&
            !ab1IsSet &&
            ( SampleMath::isPeak ( fabs( slice->at ( k - 2 ) ), fabs ( slice->at ( k ) ), fabs ( slice->at ( k + 2 ) ) ) )
        )
        {
            amplitudeb1 = slice->at ( k );
            ab1IsSet = true;
            break;
        }
    }

    if ( ! ( aa0IsSet && aa1IsSet && ab0IsSet && ab1IsSet ) )
        return false;

    if ( 	( 10 * log10 ( pow ( amplitudea1, 2 ) / pow ( amplitudea0, 2 ) ) > decibelDifference ) &&
            ( fabs ( 10 * log10 ( pow ( amplitudeb0, 2 ) / pow ( amplitudeb1, 2 ) ) ) < 0.01* decibelDifference )
       )
    {
        fprintf ( stdout, "lookaheadSamplesPlusEpsilon\t= %f lookaheadsamples\t = %f\n", lookaheadSamplesPlusEpsilon, lookaheadSamples );
        fflush ( stdout );
        fprintf ( stdout, "lookbehindSamplesPlusEpsilon\t= %f lookbehindsamples\t = %f\n", lookbehindSamplesPlusEpsilon, lookbehindSamples );
        fflush ( stdout );
        fprintf ( stdout, "amplitudea1\t= %f\t @x=: %i\n", amplitudea1, i  );
        fflush ( stdout );
        fprintf ( stdout, "amplitudea0\t= %f\t @x=: %i\n", amplitudea0, j  );
        fflush ( stdout );
        fprintf ( stdout, "dBDiffAhead\t= %f\n\n", 10 * log10 ( pow ( amplitudea1, 2 ) / pow ( amplitudea0, 2 ) ) );
        fflush ( stdout );
        fprintf ( stdout, "amplitudeb0\t= %f\t @x=: %i\n", amplitudeb0, l  );
        fflush ( stdout );
        fprintf ( stdout, "amplitudeb1\t= %f\t @x=: %i\n", amplitudeb1, k  );
        fflush ( stdout );
        fprintf ( stdout, "dBDiffBehind\t= %f\n\n\n", 10 * log10 ( pow ( amplitudeb1, 2 ) / pow ( amplitudeb0, 2 ) ) );
        fflush ( stdout );
        return true;
    }
    return false;
}

#endif
