#include <algorithm>
#include <wx/window.h>
#include "chaingrid.h"

ChainGrid::ChainGrid (
    wxWindow *parent,
    int id,
    const std::vector<wxString> &effectsVectorParameter,
    size_t sliceNo,
    int cellHeight,
    int cellWidth,
    bool verticalConstraint ) :
        MonochromeGrid ( parent, id, effectsVectorParameter.size(), sliceNo, cellHeight, cellWidth, verticalConstraint ), effectsVector ( effectsVectorParameter )
{
    SetRowLabelSize ( 0 );
    SetCellTextColour ( *wxLIGHT_GREY );

#ifndef __WXMSW__
    SetDefaultCellFont ( *wxSMALL_FONT );
#endif

    SetDefaultCellAlignment( wxALIGN_CENTRE, wxALIGN_CENTRE );
    SetColLabelTextOrientation ( wxVERTICAL );
    SetColLabelAlignment( wxALIGN_CENTRE, wxALIGN_CENTRE );
    SetColLabelSize( 30 );
    int rowNo;
    rowNo = GetNumberRows();

    for ( int i = 0; i < rowNo; i++ )
    {
        setEffectName ( i );
        setValue ( true, i, i );
    }

    wxString s;
    for ( int i = 0; i < GetNumberCols(); i++ )
    {
        s.Printf ( _("%i"), i+1 );
        SetColLabelValue ( i, s );
    }
}

void ChainGrid::getChainVector ( std::vector<wxString> *chainVector )
{
    chainVector->clear();

    int colNo, rowNo;
    colNo = GetNumberCols();
    rowNo = GetNumberRows();
    for ( int i = 0; i < colNo; i++ )
    {
        for ( int j = 0; j < rowNo; j++ )
        {
            if ( isSelected ( j, i ) )
            {
                chainVector->push_back ( effectsVector.at ( j ) );
                break;
            }
        }
    }
}

void ChainGrid::setEffectName ( int row )
{
    if ( row >= GetNumberRows() || row < 0 )
        return;

    wxString name = effectsVector.at ( row );
    int colNo = GetNumberCols();
    int nameLen = ( int ) name.size();

    if ( nameLen > colNo )
        return;

    BeginBatch();
    wxString letter;
    for ( int i = 0; i < nameLen; i++ )
    {
        letter = name.Mid ( i, 1 );
        SetCellValue ( row, i, letter );
    }
    EndBatch();
}
