#include <algorithm>

#include <wx/checkbox.h>
#include <wx/radiobox.h>
#include <wx/slider.h>
#include <wx/treebook.h>
#include <wx/textctrl.h>
#include <wx/window.h>
#include <wx/button.h>
#include <wx/tglbtn.h>
#include <wx/font.h>

#include "effectsgrid.h"
#include "chaingrid.h"
#include "enum.h"
#include "fxplotwindow.h"
#include "functor.h"
#include "settingsbook.h"
#include "smasherMain.h"

BEGIN_EVENT_TABLE( SettingsBook, wxAuiNotebook )
    EVT_IDLE( SettingsBook::OnIdle )
    EVT_RADIOBOX( ID_TREMOLO_TYPES_BOX, SettingsBook::OnRadioBoxChanged )
    EVT_RADIOBOX( ID_DEFAULT_SAMPLE_BOX, SettingsBook::OnRadioBoxChanged )
    EVT_RADIOBOX( ID_MASTER_FILTER_TYPES_BOX, SettingsBook::OnRadioBoxChanged )
    EVT_RADIOBOX( ID_MASTER_FILTER_MOD_TYPES_BOX, SettingsBook::OnRadioBoxChanged )
    EVT_RADIOBOX( ID_MASTER_FILTER_LFO_TYPES_BOX, SettingsBook::OnRadioBoxChanged )
    EVT_CHECKBOX( ID_PLAYBACK_COMPENSATION_CHECKBOX, SettingsBook::OnCheckboxChanged )
    EVT_COMMAND_SCROLL( ID_TREMOLO_DEPTH_SLIDER, SettingsBook::OnSliderChanged )
    EVT_COMMAND_SCROLL( ID_TREMOLO_PPS_SLIDER, SettingsBook::OnSliderChanged )
    EVT_COMMAND_SCROLL( ID_LOWPASS_BAND_HI_SLIDER, SettingsBook::OnSliderChanged )
    EVT_COMMAND_SCROLL( ID_LOWPASS_SLOPE_HI_SLIDER, SettingsBook::OnSliderChanged )
    EVT_COMMAND_SCROLL( ID_HIGHPASS_BAND_LO_SLIDER, SettingsBook::OnSliderChanged )
    EVT_COMMAND_SCROLL( ID_HIGHPASS_SLOPE_LO_SLIDER, SettingsBook::OnSliderChanged )
    EVT_COMMAND_SCROLL( ID_BANDPASS_BAND_LO_SLIDER, SettingsBook::OnSliderChanged )
    EVT_COMMAND_SCROLL( ID_BANDPASS_SLOPE_LO_SLIDER, SettingsBook::OnSliderChanged )
    EVT_COMMAND_SCROLL( ID_BANDPASS_BAND_HI_SLIDER, SettingsBook::OnSliderChanged )
    EVT_COMMAND_SCROLL( ID_BANDPASS_SLOPE_HI_SLIDER, SettingsBook::OnSliderChanged )
    EVT_COMMAND_SCROLL( ID_BANDSTOP_BAND_LO_SLIDER, SettingsBook::OnSliderChanged )
    EVT_COMMAND_SCROLL( ID_BANDSTOP_SLOPE_LO_SLIDER, SettingsBook::OnSliderChanged )
    EVT_COMMAND_SCROLL( ID_BANDSTOP_BAND_HI_SLIDER, SettingsBook::OnSliderChanged )
    EVT_COMMAND_SCROLL( ID_BANDSTOP_SLOPE_HI_SLIDER, SettingsBook::OnSliderChanged )
    EVT_COMMAND_SCROLL( ID_LOUDNESS_BAND_LO_SLIDER, SettingsBook::OnSliderChanged )
    EVT_COMMAND_SCROLL( ID_LOUDNESS_SLOPE_LO_SLIDER, SettingsBook::OnSliderChanged )
    EVT_COMMAND_SCROLL( ID_LOUDNESS_BAND_HI_SLIDER, SettingsBook::OnSliderChanged )
    EVT_COMMAND_SCROLL( ID_LOUDNESS_SLOPE_HI_SLIDER, SettingsBook::OnSliderChanged )
    EVT_COMMAND_SCROLL( ID_MASTER_FILTER_BAND_LO_SLIDER, SettingsBook::OnSliderChanged )
    EVT_COMMAND_SCROLL( ID_MASTER_FILTER_SLOPE_LO_SLIDER, SettingsBook::OnSliderChanged )
    EVT_COMMAND_SCROLL( ID_MASTER_FILTER_BAND_HI_SLIDER, SettingsBook::OnSliderChanged )
    EVT_COMMAND_SCROLL( ID_MASTER_FILTER_SLOPE_HI_SLIDER, SettingsBook::OnSliderChanged )
    EVT_COMMAND_SCROLL( ID_MASTER_FILTER_MOD_DEPTH_SLIDER, SettingsBook::OnSliderChanged )
    EVT_COMMAND_SCROLL( ID_MASTER_FILTER_LFO_DEPTH_SLIDER, SettingsBook::OnSliderChanged )
    EVT_COMMAND_SCROLL( ID_MASTER_FILTER_LFO_PPS_SLIDER, SettingsBook::OnSliderChanged )
    EVT_COMMAND_SCROLL( ID_MASTER_FILTER_RESONANCE_SLIDER, SettingsBook::OnSliderChanged )
    EVT_COMMAND_SCROLL( ID_PLAYBACK_COMPENSATION_SLIDER, SettingsBook::OnSliderChanged )
    EVT_TREEBOOK_PAGE_CHANGED( ID_EFFECTSTREE, SettingsBook::OnTreeChanged )
    EVT_TREEBOOK_PAGE_CHANGED( ID_GENERALTREE, SettingsBook::OnTreeChanged )
    EVT_TREEBOOK_NODE_EXPANDED( ID_EFFECTSTREE, SettingsBook::OnTreeChanged )
    EVT_TREEBOOK_NODE_EXPANDED( ID_GENERALTREE, SettingsBook::OnTreeChanged )
    EVT_AUINOTEBOOK_PAGE_CHANGED( wxID_ANY, SettingsBook::OnBookChanged )
    EVT_BUTTON( ID_DELAY_MULTITAP_PRESET1, SettingsBook::OnButton )
    EVT_BUTTON( ID_DELAY_MULTITAP_PRESET2, SettingsBook::OnButton )
    EVT_BUTTON( ID_DELAY_MULTITAP_PRESET3, SettingsBook::OnButton )
    EVT_BUTTON( ID_DELAY_MULTITAP_PRESET4, SettingsBook::OnButton )
    EVT_BUTTON( ID_DELAY_MULTITAP_PRESET5, SettingsBook::OnButton )
    EVT_BUTTON( ID_DELAY_MULTITAP_PRESET6, SettingsBook::OnButton )
    EVT_BUTTON( ID_DELAY_VARIABLE_PRESET1, SettingsBook::OnButton )
    EVT_BUTTON( ID_DELAY_VARIABLE_PRESET2, SettingsBook::OnButton )
    EVT_BUTTON( ID_DELAY_VARIABLE_PRESET3, SettingsBook::OnButton )
    EVT_BUTTON( ID_DELAY_VARIABLE_PRESET4, SettingsBook::OnButton )
    EVT_BUTTON( ID_DELAY_VARIABLE_PRESET4, SettingsBook::OnButton )
    EVT_BUTTON( ID_SHOW_FX_ALL, SettingsBook::OnButton )
    EVT_BUTTON( ID_SHOW_FX_NONE, SettingsBook::OnButton )
    EVT_BUTTON( ID_EFFECTSCHAIN_DEFAULT, SettingsBook::OnButton )
    EVT_BUTTON( ID_EFFECTSCHAIN_CLEARALL, SettingsBook::OnButton )
    EVT_SIZE( SettingsBook::OnSize )

END_EVENT_TABLE()


SettingsBook::SettingsBook ( wxWindow *parentParameter, int id, EffectsGrid *effectsGridParameter, int bookWidthParameter ) :
        wxAuiNotebook ( parentParameter, id, wxDefaultPosition, wxDefaultSize, wxAUI_NB_TOP | wxAUI_NB_SCROLL_BUTTONS ),
        parent ( parentParameter ), effectsGrid ( effectsGridParameter ), bookWidth ( bookWidthParameter ), tremoloPlotUpdateRequired ( false ), lowpassPlotUpdateRequired ( false ), highpassPlotUpdateRequired ( false ),
        bandpassPlotUpdateRequired ( false ), bandstopPlotUpdateRequired ( false ), loudnessPlotUpdateRequired ( false ),
        masterFilterPlotUpdateRequired ( false ), masterFilterModPlotUpdateRequired ( false ), masterFilterLfoPlotUpdateRequired ( false )
{
#ifndef __WXMSW__
    SetNormalFont ( *wxSMALL_FONT );
    SetSelectedFont ( *wxSMALL_FONT );
    SetMeasuringFont ( *wxSMALL_FONT );
#else
    int fontSize = wxNORMAL_FONT->GetPointSize();
    //fontSize--;
    wxFont font ( fontSize, wxNORMAL_FONT->GetFamily(),
                  wxNORMAL_FONT->GetStyle(), wxNORMAL_FONT->GetWeight() );
    SetNormalFont ( font );
    SetSelectedFont ( font );
    SetMeasuringFont ( font );
#endif

    effectsTreeWidthLeft =
#ifndef __WXMSW__
        141;
#else
        124;
#endif
    effectsTreeWidthRight = bookWidth - effectsTreeWidthLeft;
    effectsPlotHeight = 100;

    generalTreeWidthLeft =
#ifndef __WXMSW__
        120;
#else
        109;
#endif

    generalTreeWidthRight = bookWidth - generalTreeWidthLeft;
    generalPlotHeight = 100;

    fxControlSize = wxSize ( 0.9 * effectsTreeWidthRight, -1 );
    fxBoxSize = wxSize ( ( int )( 0.957 * effectsTreeWidthRight ), -1 );
    fxPlotSize = wxSize ( ( int )( 0.9 * effectsTreeWidthRight ), ( int ) ( effectsPlotHeight ) );

    generalPlotSize = wxSize ( ( int ) ( 0.9 * generalTreeWidthRight ), ( int ) ( generalPlotHeight ) );
    generalControlSize = wxSize ( ( int ) ( 0.9 * generalTreeWidthRight ), -1 );
    generalBoxSize = wxSize ( ( int ) ( 0.95 * generalTreeWidthRight ), -1 );

    generalTree = new wxTreebook ( this, ID_GENERALTREE, wxDefaultPosition, wxDefaultSize, wxBK_LEFT );

    wxPanel *keyMapperPanel;
    keyMapperPanel = new wxPanel ( generalTree, wxID_ANY );
    keyTextCtrl = new wxTextCtrl ( keyMapperPanel, wxID_ANY, _("smasher"), wxDefaultPosition, generalControlSize );
    wxStaticBoxSizer *keyBoxSizer;
    keyBoxSizer = new wxStaticBoxSizer ( wxVERTICAL, keyMapperPanel, _("key") );
    keyBoxSizer->SetMinSize ( generalBoxSize );
    keyBoxSizer->Add ( keyTextCtrl );
    keyMapperPanel->SetSizer ( keyBoxSizer );
    keyBoxSizer->Layout();

    wxPanel *gatePanel;
    gatePanel = new wxPanel ( generalTree, wxID_ANY );
    gateCheckbox = new wxCheckBox ( gatePanel, ID_GATE_BOX, _("enable gate") );
    wxStaticBoxSizer *gateSizer;
    gateSizer = new wxStaticBoxSizer ( wxVERTICAL, gatePanel, _("") );
    gateSizer->SetMinSize (generalBoxSize ) ;
    gateSizer->AddSpacer ( 5 );
    gateSizer->Add ( gateCheckbox );
    gatePanel->SetSizer ( gateSizer );
    gateSizer->Layout();

    wxPanel *pitchAdjustmentPanel;
    pitchAdjustmentPanel = new wxPanel ( generalTree, wxID_ANY );

    pitchAdjustCheckbox = new wxCheckBox ( pitchAdjustmentPanel, ID_PITCH_ADJUST_BOX, _("adjust pitch") );
    pitchSemitonesSlider = new wxSlider ( pitchAdjustmentPanel, wxID_ANY,
                                          0, -12, 12, wxDefaultPosition, generalControlSize,
                                          wxSL_LABELS | wxSL_HORIZONTAL | wxSL_AUTOTICKS );
    pitchSemitonesSlider->SetTickFreq ( 1, -1 );
    pitchSemitonesSlider->SetLineSize ( 1 );
    wxStaticBoxSizer *pitchSemitonesSizer;
    pitchSemitonesSizer = new wxStaticBoxSizer ( wxVERTICAL, pitchAdjustmentPanel, _("semitones") );
    pitchSemitonesSizer->SetMinSize ( generalBoxSize );
    pitchSemitonesSizer->Add ( pitchSemitonesSlider );
    wxBoxSizer *pitchAdjustmentSizer;
    pitchAdjustmentSizer = new wxBoxSizer ( wxVERTICAL );
    pitchAdjustmentSizer->Add ( pitchSemitonesSizer );
    pitchAdjustmentSizer->AddSpacer ( 10 );
    pitchAdjustmentSizer->Add ( pitchAdjustCheckbox );
    pitchAdjustmentPanel->SetSizer ( pitchAdjustmentSizer );
    pitchSemitonesSizer->Layout();

    wxPanel *normalizePanel;
    normalizePanel = new wxPanel ( generalTree, wxID_ANY );
    normalizeSlider = new wxSlider ( normalizePanel, wxID_ANY,
                                     99, 0, 100, wxDefaultPosition, generalControlSize,
                                     wxSL_LABELS | wxSL_HORIZONTAL | wxSL_AUTOTICKS );
    normalizeSlider->SetTickFreq ( 10, -1 );
    normalizeSlider->SetLineSize ( 5 );
    normalizeSlider->SetPageSize ( 10 );
    normalizeCheckbox = new wxCheckBox ( normalizePanel, ID_NORMALIZE_BOX, _("enable normalize") );
    normalizeCheckbox->SetValue( false );
    wxStaticBoxSizer *normalizeSliderSizer;
    normalizeSliderSizer = new wxStaticBoxSizer ( wxVERTICAL, normalizePanel, _("level (%)"));
    normalizeSliderSizer->SetMinSize ( generalBoxSize );
    normalizeSliderSizer->Add ( normalizeSlider );
    wxBoxSizer *normalizeSizer;
    normalizeSizer = new wxBoxSizer ( wxVERTICAL );
    normalizeSizer->Add ( normalizeSliderSizer );
    normalizeSizer->AddSpacer ( 10 );
    normalizeSizer->Add ( normalizeCheckbox );
    normalizePanel->SetSizer ( normalizeSizer );
    normalizeSizer->Layout();

    wxPanel *compressPanel;
    compressPanel = new wxPanel ( generalTree, wxID_ANY );
    compressCutoffSlider = new wxSlider ( compressPanel, wxID_ANY,
                                          20, 0, 100, wxDefaultPosition, generalControlSize,
                                          wxSL_LABELS | wxSL_HORIZONTAL | wxSL_AUTOTICKS );
    compressCutoffSlider->SetTickFreq ( 10, -1 );
    compressCutoffSlider->SetLineSize ( 5 );
    compressCutoffSlider->SetPageSize ( 10 );
    compressRangeSlider = new wxSlider ( compressPanel, wxID_ANY,
                                         2, 0, 100, wxDefaultPosition, generalControlSize,
                                         wxSL_LABELS | wxSL_HORIZONTAL | wxSL_AUTOTICKS );
    compressRangeSlider->SetTickFreq ( 10, -1 );
    compressRangeSlider->SetLineSize ( 5 );
    compressRangeSlider->SetPageSize ( 10 );
    compressCheckbox = new wxCheckBox ( compressPanel, ID_COMPRESS_BOX, _("enable compress") );
    compressCheckbox->SetValue( false );
    wxStaticBoxSizer *compressCutoffSizer, *compressRangeSizer;
    compressCutoffSizer = new wxStaticBoxSizer ( wxVERTICAL, compressPanel, _("cutoff (%)") );
    compressCutoffSizer->SetMinSize ( generalBoxSize );
    compressCutoffSizer->Add ( compressCutoffSlider );
    compressRangeSizer = new wxStaticBoxSizer ( wxVERTICAL, compressPanel, _("range (%)") );
    compressRangeSizer->SetMinSize ( generalBoxSize );
    compressRangeSizer->Add ( compressRangeSlider );
    wxBoxSizer *compressSizer;
    compressSizer = new wxBoxSizer ( wxVERTICAL );
    compressSizer->Add ( compressCutoffSizer );
    compressSizer->Add ( compressRangeSizer );
    compressSizer->AddSpacer ( 10 );
    compressSizer->Add ( compressCheckbox );
    compressPanel->SetSizer ( compressSizer );
    compressSizer->Layout();

    wxPanel *loudnessPanel;
    loudnessPanel = new wxPanel ( generalTree, wxID_ANY );

    loudnessBandLoSlider = new wxSlider ( loudnessPanel, ID_LOUDNESS_BAND_LO_SLIDER,
                                          4000, 0, 22050, wxDefaultPosition, generalControlSize,
                                          wxSL_LABELS | wxSL_HORIZONTAL | wxSL_AUTOTICKS );
    loudnessBandLoSlider->SetTickFreq ( 2000, -1 );
    loudnessBandLoSlider->SetLineSize ( 100 );
    loudnessBandLoSlider->SetPageSize ( 1000 );

    loudnessSlopeLoSlider = new wxSlider ( loudnessPanel, ID_LOUDNESS_SLOPE_LO_SLIDER,
                                           1500, 0, 22050, wxDefaultPosition, generalControlSize,
                                           wxSL_LABELS | wxSL_HORIZONTAL | wxSL_AUTOTICKS );
    loudnessSlopeLoSlider->SetTickFreq ( 2000, -1 );
    loudnessSlopeLoSlider->SetLineSize ( 100 );
    loudnessSlopeLoSlider->SetPageSize ( 1000 );

    loudnessBandHiSlider = new wxSlider ( loudnessPanel, ID_LOUDNESS_BAND_HI_SLIDER,
                                          8000, 0, 22050, wxDefaultPosition, generalControlSize,
                                          wxSL_LABELS | wxSL_HORIZONTAL | wxSL_AUTOTICKS );
    loudnessBandHiSlider->SetTickFreq ( 2000, -1 );
    loudnessBandHiSlider->SetLineSize ( 100 );
    loudnessBandHiSlider->SetPageSize ( 1000 );

    loudnessSlopeHiSlider = new wxSlider ( loudnessPanel, ID_LOUDNESS_SLOPE_LO_SLIDER,
                                           1500, 0, 22050, wxDefaultPosition, generalControlSize,
                                           wxSL_LABELS | wxSL_HORIZONTAL | wxSL_AUTOTICKS );
    loudnessSlopeHiSlider->SetTickFreq ( 2000, -1 );
    loudnessSlopeHiSlider->SetLineSize ( 100 );
    loudnessSlopeHiSlider->SetPageSize ( 1000 );

    loudnessPlot = new FxPlotWindow ( loudnessPanel, wxID_ANY, wxPoint ( 0, 0 ), generalPlotSize );


    wxStaticBoxSizer *loudnessBandLoSizer, *loudnessSlopeLoSizer,
    *loudnessBandHiSizer, *loudnessSlopeHiSizer, *loudnessPlotSizer;
    loudnessBandLoSizer = new wxStaticBoxSizer ( wxVERTICAL, loudnessPanel, _("cutoff low (hz)") );
    loudnessSlopeLoSizer = new wxStaticBoxSizer ( wxVERTICAL, loudnessPanel, _("slope low (hz)") );
    loudnessBandHiSizer = new wxStaticBoxSizer ( wxVERTICAL, loudnessPanel, _("cutoff high (hz)") );
    loudnessSlopeHiSizer = new wxStaticBoxSizer ( wxVERTICAL, loudnessPanel, _("slope high (hz)") );
    loudnessPlotSizer = new wxStaticBoxSizer ( wxVERTICAL, loudnessPanel, _("preview") );

    loudnessBandLoSizer->SetMinSize ( generalBoxSize );
    loudnessSlopeLoSizer->SetMinSize ( generalBoxSize );
    loudnessBandHiSizer->SetMinSize ( generalBoxSize );
    loudnessSlopeHiSizer->SetMinSize ( generalBoxSize );
    loudnessPlotSizer->SetMinSize ( generalBoxSize );

    loudnessBandLoSizer->Add ( loudnessBandLoSlider );
    loudnessSlopeLoSizer->Add ( loudnessSlopeLoSlider );
    loudnessBandHiSizer->Add ( loudnessBandHiSlider );
    loudnessSlopeHiSizer->Add ( loudnessSlopeHiSlider );
    loudnessPlotSizer->Add ( loudnessPlot );

    loudnessCheckbox = new wxCheckBox ( loudnessPanel, ID_LOUDNESS_BOX, _( "enable loudness" ) );
    loudnessCheckbox->SetValue( false );

    wxBoxSizer *loudnessSizer;
    loudnessSizer = new wxBoxSizer ( wxVERTICAL );
    loudnessSizer->Add ( loudnessBandLoSizer ) ;
    loudnessSizer->Add ( loudnessSlopeLoSizer ) ;
    loudnessSizer->Add ( loudnessBandHiSizer ) ;
    loudnessSizer->Add ( loudnessSlopeHiSizer ) ;
    loudnessSizer->Add ( loudnessPlotSizer );
    loudnessSizer->AddSpacer ( 10 );
    loudnessSizer->Add ( loudnessCheckbox );
    loudnessPanel->SetSizer ( loudnessSizer );
    loudnessSizer->Layout();

    wxPanel *masterFilterPanel, *masterFilterModPanel, *masterFilterLfoPanel, *masterFilterResPanel;
    masterFilterPanel = new wxPanel ( generalTree, wxID_ANY );
    masterFilterModPanel = new wxPanel ( generalTree, wxID_ANY );
    masterFilterLfoPanel = new wxPanel ( generalTree, wxID_ANY );
    masterFilterResPanel = new wxPanel ( generalTree, wxID_ANY );

    wxString masterFilterTypesChoices[] = { _( "low pass" ), _( "high pass" ), _( "band pass" ), _( "band stop" ) };
    masterFilterTypesBox = new wxRadioBox ( masterFilterPanel, ID_MASTER_FILTER_TYPES_BOX, _("type"),
                                            wxDefaultPosition, generalControlSize, 4, masterFilterTypesChoices, 4, wxRA_SPECIFY_ROWS );
    masterFilterTypesBox->SetSelection ( 0 );
    masterFilterTypesBox->SetMinSize ( generalBoxSize );

    masterFilterBandLoSlider = new wxSlider ( masterFilterPanel, ID_MASTER_FILTER_BAND_LO_SLIDER,
            6000, 0, 22050, wxDefaultPosition, generalControlSize,
            wxSL_LABELS | wxSL_HORIZONTAL | wxSL_AUTOTICKS );
    masterFilterBandLoSlider->SetTickFreq ( 2000, -1 );
    masterFilterBandLoSlider->SetLineSize ( 100 );
    masterFilterBandLoSlider->SetPageSize ( 1000 );

    masterFilterSlopeLoSlider = new wxSlider ( masterFilterPanel, ID_MASTER_FILTER_SLOPE_LO_SLIDER,
            1000, 0, 22050, wxDefaultPosition, generalControlSize,
            wxSL_LABELS | wxSL_HORIZONTAL | wxSL_AUTOTICKS );
    masterFilterSlopeLoSlider->SetTickFreq ( 2000, -1 );
    masterFilterSlopeLoSlider->SetLineSize ( 100 );
    masterFilterSlopeLoSlider->SetPageSize ( 1000 );

    masterFilterBandHiSlider = new wxSlider ( masterFilterPanel, ID_MASTER_FILTER_BAND_HI_SLIDER,
            6000, 0, 22050, wxDefaultPosition, generalControlSize,
            wxSL_LABELS | wxSL_HORIZONTAL | wxSL_AUTOTICKS );
    masterFilterBandHiSlider->SetTickFreq ( 2000, -1 );
    masterFilterBandHiSlider->SetLineSize ( 100 );
    masterFilterBandHiSlider->SetPageSize ( 1000 );

    masterFilterSlopeHiSlider = new wxSlider ( masterFilterPanel, ID_MASTER_FILTER_SLOPE_HI_SLIDER,
            1000, 0, 22050, wxDefaultPosition, generalControlSize,
            wxSL_LABELS | wxSL_HORIZONTAL | wxSL_AUTOTICKS );
    masterFilterSlopeHiSlider->SetTickFreq ( 2000, -1 );
    masterFilterSlopeHiSlider->SetLineSize ( 100 );
    masterFilterSlopeHiSlider->SetPageSize ( 1000 );

    masterFilterCheckbox = new wxCheckBox ( masterFilterPanel, ID_MASTER_FILTER_BOX, _( "enable master filter" ) );
    masterFilterCheckbox->SetValue( false );

    masterFilterPlot = new FxPlotWindow ( masterFilterPanel, wxID_ANY, wxPoint ( 0, 0 ), generalPlotSize );

    wxStaticBoxSizer *masterFilterBandLoSizer, *masterFilterSlopeLoSizer, *masterFilterBandHiSizer, *masterFilterSlopeHiSizer, *masterFilterPlotSizer;
    masterFilterBandLoSizer = new wxStaticBoxSizer ( wxVERTICAL, masterFilterPanel, _("cutoff low (hz)") );
    masterFilterBandLoSizer->SetMinSize ( generalBoxSize );
    masterFilterBandLoSizer->Add ( masterFilterBandLoSlider );
    masterFilterSlopeLoSizer = new wxStaticBoxSizer ( wxVERTICAL, masterFilterPanel, _("slope low (hz)") );
    masterFilterSlopeLoSizer->SetMinSize ( generalBoxSize );
    masterFilterSlopeLoSizer->Add ( masterFilterSlopeLoSlider );
    masterFilterBandHiSizer = new wxStaticBoxSizer ( wxVERTICAL, masterFilterPanel, _("cutoff high (hz)") );
    masterFilterBandHiSizer->SetMinSize ( generalBoxSize );
    masterFilterBandHiSizer->Add ( masterFilterBandHiSlider );
    masterFilterSlopeHiSizer = new wxStaticBoxSizer ( wxVERTICAL, masterFilterPanel, _("slope high (hz)") );
    masterFilterSlopeHiSizer->SetMinSize ( generalBoxSize );
    masterFilterSlopeHiSizer->Add ( masterFilterSlopeHiSlider );
    masterFilterPlotSizer = new wxStaticBoxSizer ( wxVERTICAL, masterFilterPanel, _("preview") );
    masterFilterPlotSizer->SetMinSize ( generalBoxSize );
    masterFilterPlotSizer->Add( masterFilterPlot );

    wxBoxSizer *masterFilterSizer;
    masterFilterSizer = new wxBoxSizer ( wxVERTICAL );
    masterFilterSizer->Add ( masterFilterTypesBox ) ;
    masterFilterSizer->Add ( masterFilterBandLoSizer ) ;
    masterFilterSizer->Add ( masterFilterSlopeLoSizer ) ;
    masterFilterSizer->Add ( masterFilterBandHiSizer ) ;
    masterFilterSizer->Add ( masterFilterSlopeHiSizer ) ;
    masterFilterSizer->Add ( masterFilterPlotSizer );
    masterFilterSizer->AddSpacer ( 10 );
    masterFilterSizer->Add ( masterFilterCheckbox );
    masterFilterSizer->Layout();


    masterFilterResonanceSlider = new wxSlider ( masterFilterResPanel, ID_MASTER_FILTER_RESONANCE_SLIDER,
            2, 1, 20, wxDefaultPosition, fxControlSize,
            wxSL_LABELS | wxSL_HORIZONTAL | wxSL_AUTOTICKS );
    masterFilterResonanceSlider->SetTickFreq ( 5, -1 );
    masterFilterResonanceSlider->SetLineSize ( 1 );
    masterFilterResonanceSlider->SetPageSize ( 2 );
    masterFilterResonanceSlider->SetMinSize ( generalControlSize );

    masterFilterResonanceCheckbox = new wxCheckBox ( masterFilterResPanel, ID_MASTER_FILTER_RESONANCE_BOX, _("enable resonance") );
    masterFilterResonanceCheckbox->SetValue( false );

    wxStaticBoxSizer *masterFilterResonanceSliderSizer;
    masterFilterResonanceSliderSizer = new wxStaticBoxSizer ( wxVERTICAL, masterFilterResPanel, _("gain") );
    masterFilterResonanceSliderSizer->SetMinSize ( generalBoxSize  );
    masterFilterResonanceSliderSizer->Add ( masterFilterResonanceSlider );

    wxBoxSizer *masterFilterResSizer;
    masterFilterResSizer = new wxBoxSizer ( wxVERTICAL );
    masterFilterResSizer->Add ( masterFilterResonanceSliderSizer );
    masterFilterResSizer->AddSpacer ( 10 );
    masterFilterResSizer->Add ( masterFilterResonanceCheckbox );
    masterFilterResPanel->SetSizer ( masterFilterResSizer );
    masterFilterResSizer->Layout();


    wxString masterFilterModulationTypesChoices[] =
        { _( "in" ), _( "out" ) };
    masterFilterModulationTypesBox = new wxRadioBox ( masterFilterModPanel, ID_MASTER_FILTER_MOD_TYPES_BOX, _("types"),
            wxDefaultPosition, generalControlSize, 2, masterFilterModulationTypesChoices, 2, wxRA_SPECIFY_ROWS );
    masterFilterModulationTypesBox->SetSelection ( 0 );
    masterFilterModulationTypesBox->SetMinSize ( generalBoxSize );

    masterFilterModulationDepthSlider = new wxSlider ( masterFilterModPanel, ID_MASTER_FILTER_MOD_DEPTH_SLIDER,
            100, 0, 100, wxDefaultPosition, generalControlSize,
            wxSL_LABELS | wxSL_HORIZONTAL | wxSL_AUTOTICKS );
    masterFilterModulationDepthSlider->SetTickFreq ( 10, -1 );
    masterFilterModulationDepthSlider->SetLineSize ( 5 );
    masterFilterModulationDepthSlider->SetPageSize ( 10 );
    masterFilterModulationDepthSlider->SetMinSize ( generalControlSize );

    masterFilterModPlot = new FxPlotWindow ( masterFilterModPanel, wxID_ANY, wxPoint ( 0, 0 ), generalPlotSize );

    masterFilterModulationCheckbox = new wxCheckBox ( masterFilterModPanel, ID_MASTER_FILTER_MOD_BOX, _("enable sweep") );
    masterFilterModulationCheckbox->SetValue( false );

    wxStaticBoxSizer *masterFilterModulationDepthSizer, *masterFilterModPlotSizer;
    masterFilterModulationDepthSizer = new wxStaticBoxSizer ( wxVERTICAL, masterFilterModPanel, _("depth (%)") );
    masterFilterModulationDepthSizer->SetMinSize ( generalBoxSize );
    masterFilterModulationDepthSizer->Add ( masterFilterModulationDepthSlider );

    masterFilterModPlotSizer = new wxStaticBoxSizer ( wxVERTICAL, masterFilterModPanel, _("preview") );
    masterFilterModPlotSizer->SetMinSize ( generalBoxSize );
    masterFilterModPlotSizer->Add( masterFilterModPlot );

    wxBoxSizer *masterFilterModSizer;
    masterFilterModSizer = new wxBoxSizer ( wxVERTICAL );
    masterFilterModSizer->Add ( masterFilterModulationTypesBox ) ;
    masterFilterModSizer->Add ( masterFilterModulationDepthSizer );
    masterFilterModSizer->Add ( masterFilterModPlotSizer );
    masterFilterModSizer->AddSpacer ( 10 );
    masterFilterModSizer->Add ( masterFilterModulationCheckbox );
    masterFilterModPanel->SetSizer ( masterFilterModSizer );
    masterFilterModSizer->Layout();


    wxString masterFilterLfoTypesChoices[] =
        { _("sine squared"), _("sine"), _("sawtooth"), _("square"), _("triangle"), _("sample and hold") };
    masterFilterLfoTypesBox = new wxRadioBox ( masterFilterLfoPanel, ID_MASTER_FILTER_LFO_TYPES_BOX, _("types"),
            wxDefaultPosition, generalControlSize, 6, masterFilterLfoTypesChoices, 6, wxRA_SPECIFY_ROWS );
    masterFilterLfoTypesBox->SetSelection ( 0 );
    masterFilterLfoTypesBox->SetMinSize ( generalBoxSize );

    masterFilterLfoDepthSlider = new wxSlider ( masterFilterLfoPanel, ID_MASTER_FILTER_LFO_DEPTH_SLIDER,
            100, 0, 100, wxDefaultPosition, generalControlSize,
            wxSL_LABELS | wxSL_HORIZONTAL | wxSL_AUTOTICKS );
    masterFilterLfoDepthSlider->SetTickFreq ( 10, -1 );
    masterFilterLfoDepthSlider->SetLineSize ( 1 );
    masterFilterLfoDepthSlider->SetPageSize ( 5 );
    masterFilterLfoDepthSlider->SetMinSize ( generalControlSize );

    masterFilterLfoPeriodsPerSliceSlider = new wxSlider ( masterFilterLfoPanel, ID_MASTER_FILTER_LFO_PPS_SLIDER,
            2, 1, 40, wxDefaultPosition, fxControlSize,
            wxSL_LABELS | wxSL_HORIZONTAL | wxSL_AUTOTICKS );
    masterFilterLfoPeriodsPerSliceSlider->SetTickFreq ( 10, -1 );
    masterFilterLfoPeriodsPerSliceSlider->SetLineSize ( 1 );
    masterFilterLfoPeriodsPerSliceSlider->SetPageSize ( 1 );
    masterFilterLfoPeriodsPerSliceSlider->SetMinSize ( generalControlSize );

    masterFilterLfoPlot = new FxPlotWindow ( masterFilterLfoPanel, wxID_ANY, wxPoint ( 0, 0 ), generalPlotSize );

    masterFilterLfoCheckbox = new wxCheckBox ( masterFilterLfoPanel, ID_MASTER_FILTER_LFO_BOX, _( "enable lfo" ) );
    masterFilterLfoCheckbox->SetValue( false );

    wxStaticBoxSizer *masterFilterLfoDepthSizer, *masterFilterLfoPeriodsPerSliceSliderSizer, *masterFilterLfoPlotSizer;

    masterFilterLfoDepthSizer = new wxStaticBoxSizer ( wxVERTICAL, masterFilterLfoPanel, _("depth (%)"));
    masterFilterLfoDepthSizer->Add( masterFilterLfoDepthSlider );
    masterFilterLfoDepthSizer->SetMinSize ( generalBoxSize );

    masterFilterLfoPeriodsPerSliceSliderSizer = new wxStaticBoxSizer ( wxVERTICAL, masterFilterLfoPanel, _("periods"));
    masterFilterLfoPeriodsPerSliceSliderSizer->SetMinSize ( generalBoxSize  );
    masterFilterLfoPeriodsPerSliceSliderSizer->Add( masterFilterLfoPeriodsPerSliceSlider );

    masterFilterLfoPlotSizer = new wxStaticBoxSizer ( wxVERTICAL, masterFilterLfoPanel, _("preview"));
    masterFilterLfoPlotSizer->SetMinSize ( generalBoxSize );
    masterFilterLfoPlotSizer->Add( masterFilterLfoPlot );

    wxBoxSizer *masterFilterLfoSizer;
    masterFilterLfoSizer = new wxBoxSizer ( wxVERTICAL );
    masterFilterLfoSizer->Add ( masterFilterLfoTypesBox ) ;
    masterFilterLfoSizer->Add ( masterFilterLfoDepthSizer );
    masterFilterLfoSizer->Add ( masterFilterLfoPeriodsPerSliceSliderSizer );
    masterFilterLfoSizer->Add ( masterFilterLfoPlotSizer );
    masterFilterLfoSizer->AddSpacer ( 10 );
    masterFilterLfoSizer->Add ( masterFilterLfoCheckbox );
    masterFilterLfoPanel->SetSizer ( masterFilterLfoSizer );
    masterFilterLfoSizer->Layout();

    effectsTree = new wxTreebook ( this, ID_EFFECTSTREE, wxDefaultPosition, wxDefaultSize,
                                   wxBK_LEFT );
    wxPanel *showFxPanel = new wxPanel ( effectsTree, wxID_ANY );
    wxString fxName;
    wxToggleButton *showFxButton;
    wxStaticBoxSizer *showFxSizer;
    showFxSizer = new wxStaticBoxSizer ( wxVERTICAL, showFxPanel, _(""));
    showFxSizer->SetMinSize( fxBoxSize );
    size_t rowNo = effectsGrid->GetNumberRows();
    int showFxId = ID_EFFECTS_START;
    size_t i;
    for ( i = 0; i < rowNo; i++ )
    {
        fxName = effectsGrid->GetRowLabelValue( i );
        showFxId = ID_EFFECTS_START + i;
        showFxButton = new wxToggleButton ( showFxPanel, showFxId, fxName, wxDefaultPosition, wxDefaultSize, 2 );

        bool b = ( fxName != _T("fade in") && fxName != _T("fade out") && fxName != _T("bandpass") && fxName != _T("bandstop") );

        showFxButton->SetValue ( b );
        showFxButton->SetMinSize (
#ifdef __WXMSW__
            fxBoxSize
#else
            fxControlSize
#endif
            );
        showFxSizer->Add ( showFxButton );
    }

    Connect ( ID_EFFECTS_START, showFxId, wxEVT_COMMAND_TOGGLEBUTTON_CLICKED, wxCommandEventHandler( SettingsBook::OnToggleButtonChanged ) );

    showFxAllButton = new wxButton ( showFxPanel, ID_SHOW_FX_ALL, _T ( "show all"), wxDefaultPosition, fxControlSize );
    showFxNoneButton = new wxButton ( showFxPanel, ID_SHOW_FX_NONE, _T ( "hide all"), wxDefaultPosition, fxControlSize );

    showFxSizer->AddSpacer ( 10 );
    showFxSizer->Add ( showFxAllButton );
    showFxSizer->Add ( showFxNoneButton );

    showFxPanel->SetSizer ( showFxSizer );
    showFxSizer->Layout();

    wxPanel *delayMultiTapPanel;
    delayMultiTapPanel = new wxPanel ( effectsTree, wxID_ANY );

    delayMultitapPreset1Button = new wxButton ( delayMultiTapPanel, ID_DELAY_MULTITAP_PRESET1, _T ( "1/100"), wxDefaultPosition, fxControlSize );
    delayMultitapPreset2Button = new wxButton ( delayMultiTapPanel, ID_DELAY_MULTITAP_PRESET2, _T ( "1/50"), wxDefaultPosition, fxControlSize );
    delayMultitapPreset3Button = new wxButton ( delayMultiTapPanel, ID_DELAY_MULTITAP_PRESET3, _T ( "1/8"), wxDefaultPosition, fxControlSize );
    delayMultitapPreset4Button = new wxButton ( delayMultiTapPanel, ID_DELAY_MULTITAP_PRESET4, _T ( "1/4"), wxDefaultPosition, fxControlSize );
    delayMultitapPreset5Button = new wxButton ( delayMultiTapPanel, ID_DELAY_MULTITAP_PRESET5, _T ( "1/3"), wxDefaultPosition, fxControlSize );
    delayMultitapPreset6Button = new wxButton ( delayMultiTapPanel, ID_DELAY_MULTITAP_PRESET6, _T ( "1/2"), wxDefaultPosition, fxControlSize );

    delayMultiTapDistanceSlider = new wxSlider ( delayMultiTapPanel, wxID_ANY,
            1, 1, 100, wxDefaultPosition, fxControlSize,
            wxSL_LABELS | wxSL_HORIZONTAL | wxSL_AUTOTICKS );
    delayMultiTapDistanceSlider->SetTickFreq ( 10, -1 );
    delayMultiTapDistanceSlider->SetLineSize ( 1 );
    delayMultiTapDistanceSlider->SetPageSize ( 10 );

    delayMultiTapFeedbackSlider = new wxSlider ( delayMultiTapPanel, wxID_ANY,
            0, 0, 10, wxDefaultPosition, fxControlSize,
            wxSL_LABELS | wxSL_HORIZONTAL | wxSL_AUTOTICKS );
    delayMultiTapFeedbackSlider->SetTickFreq ( 1, -1 );
    delayMultiTapFeedbackSlider->SetLineSize ( 1 );
    delayMultiTapFeedbackSlider->SetPageSize ( 2 );

    delayMultiTapGainSlider = new wxSlider ( delayMultiTapPanel, wxID_ANY,
            1, 0, 10, wxDefaultPosition, fxControlSize,
            wxSL_LABELS | wxSL_HORIZONTAL | wxSL_AUTOTICKS );
    delayMultiTapGainSlider->SetTickFreq ( 1, -1 );
    delayMultiTapGainSlider->SetLineSize ( 1 );
    delayMultiTapGainSlider->SetPageSize ( 2 );

    delayMultiTapStepNoSlider = new wxSlider ( delayMultiTapPanel, wxID_ANY,
            1, 1, 4, wxDefaultPosition, fxControlSize,
            wxSL_LABELS | wxSL_HORIZONTAL | wxSL_AUTOTICKS );
    delayMultiTapStepNoSlider->SetTickFreq ( 1, -1 );
    delayMultiTapStepNoSlider->SetLineSize ( 1 );
    delayMultiTapStepNoSlider->SetPageSize ( 1 );

    wxStaticBoxSizer *delayMultiTapFeedbackSliderSizer, *delayMultiTapStepNoSliderSizer, *delayMultiTapDistanceSliderSizer, *delayMultiTapGainSliderSizer;

    delayMultiTapFeedbackSliderSizer = new wxStaticBoxSizer ( wxVERTICAL, delayMultiTapPanel, _("feedback"));
    delayMultiTapFeedbackSliderSizer->SetMinSize( fxBoxSize );
    delayMultiTapFeedbackSliderSizer->Add ( delayMultiTapFeedbackSlider );

    delayMultiTapStepNoSliderSizer = new wxStaticBoxSizer ( wxVERTICAL, delayMultiTapPanel, _("taps"));
    delayMultiTapStepNoSliderSizer->SetMinSize( fxBoxSize );
    delayMultiTapStepNoSliderSizer->Add ( delayMultiTapStepNoSlider );

    delayMultiTapDistanceSliderSizer = new wxStaticBoxSizer ( wxVERTICAL, delayMultiTapPanel, _("distance (%)"));
    delayMultiTapDistanceSliderSizer->SetMinSize( fxBoxSize );
    delayMultiTapDistanceSliderSizer->AddSpacer ( 5 );
    delayMultiTapDistanceSliderSizer->Add ( delayMultiTapDistanceSlider );
    delayMultiTapDistanceSliderSizer->Add ( delayMultitapPreset1Button );
    delayMultiTapDistanceSliderSizer->Add ( delayMultitapPreset2Button );
    delayMultiTapDistanceSliderSizer->Add ( delayMultitapPreset3Button );
    delayMultiTapDistanceSliderSizer->Add ( delayMultitapPreset4Button );
    delayMultiTapDistanceSliderSizer->Add ( delayMultitapPreset5Button );
    delayMultiTapDistanceSliderSizer->Add ( delayMultitapPreset6Button );


    delayMultiTapGainSliderSizer = new wxStaticBoxSizer ( wxVERTICAL, delayMultiTapPanel, _("gain"));
    delayMultiTapGainSliderSizer->SetMinSize( fxBoxSize );
    delayMultiTapGainSliderSizer->Add ( delayMultiTapGainSlider );

    wxBoxSizer *delayMultiTapSizer;

    delayMultiTapSizer = new wxBoxSizer ( wxVERTICAL );
    delayMultiTapSizer->Add ( delayMultiTapDistanceSliderSizer );
    delayMultiTapSizer->Add ( delayMultiTapGainSliderSizer );
    delayMultiTapSizer->Add( delayMultiTapFeedbackSliderSizer );
    delayMultiTapSizer->Add( delayMultiTapStepNoSliderSizer );
    delayMultiTapPanel->SetSizer ( delayMultiTapSizer );
    delayMultiTapSizer->Layout();


    wxPanel *delayVariablePanel;
    delayVariablePanel = new wxPanel ( effectsTree, wxID_ANY );

    delayVariablePreset1Button = new wxButton ( delayVariablePanel, ID_DELAY_VARIABLE_PRESET1, _T ( "32 samples"), wxDefaultPosition, fxControlSize );
    delayVariablePreset2Button = new wxButton ( delayVariablePanel, ID_DELAY_VARIABLE_PRESET2, _T ( "64 samples"), wxDefaultPosition, fxControlSize );
    delayVariablePreset3Button = new wxButton ( delayVariablePanel, ID_DELAY_VARIABLE_PRESET3, _T ( "128 samples"), wxDefaultPosition, fxControlSize );
    delayVariablePreset4Button = new wxButton ( delayVariablePanel, ID_DELAY_VARIABLE_PRESET4, _T ( "256 samples"), wxDefaultPosition, fxControlSize );

    delayVariableDistanceSlider = new wxSlider ( delayVariablePanel, wxID_ANY,
            1, 1, 256, wxDefaultPosition, fxControlSize,
            wxSL_LABELS | wxSL_HORIZONTAL | wxSL_AUTOTICKS );
    delayVariableDistanceSlider->SetTickFreq ( 64, -1 );
    delayVariableDistanceSlider->SetLineSize ( 1 );
    delayVariableDistanceSlider->SetPageSize ( 1 );

    delayVariableFeedbackSlider = new wxSlider ( delayVariablePanel, wxID_ANY,
            0, 0, 10, wxDefaultPosition, fxControlSize,
            wxSL_LABELS | wxSL_HORIZONTAL | wxSL_AUTOTICKS );
    delayVariableFeedbackSlider->SetTickFreq ( 1, -1 );
    delayVariableFeedbackSlider->SetLineSize ( 1 );
    delayVariableFeedbackSlider->SetPageSize ( 2 );

    delayVariableGainSlider = new wxSlider ( delayVariablePanel, wxID_ANY,
            1, 0, 10, wxDefaultPosition, fxControlSize,
            wxSL_LABELS | wxSL_HORIZONTAL | wxSL_AUTOTICKS );
    delayVariableGainSlider->SetTickFreq ( 1, -1 );
    delayVariableGainSlider->SetLineSize ( 1 );
    delayVariableGainSlider->SetPageSize ( 2 );

    wxString delayLfoTypesChoices[] = {_("sine squared"), _("triangle")};
    delayVariableLfoTypesBox = new wxRadioBox ( delayVariablePanel, wxID_ANY, _("lfo type"),
            wxDefaultPosition, fxBoxSize, 2, delayLfoTypesChoices, 2, wxRA_SPECIFY_ROWS);
    delayVariableLfoTypesBox->SetSelection( 0 );
    delayVariableLfoTypesBox->SetMinSize( fxBoxSize );

    delayVariableLfoDepthSlider = new wxSlider ( delayVariablePanel, wxID_ANY,
            1, 0, 100, wxDefaultPosition, fxControlSize,
            wxSL_LABELS | wxSL_HORIZONTAL | wxSL_AUTOTICKS );
    delayVariableLfoDepthSlider->SetTickFreq ( 10, -1 );
    delayVariableLfoDepthSlider->SetLineSize ( 1 );
    delayVariableLfoDepthSlider->SetPageSize ( 10 );

    delayVariableLfoPeriodsPerSliceSlider = new wxSlider ( delayVariablePanel, wxID_ANY,
            25, 1, 100, wxDefaultPosition, fxControlSize,
            wxSL_LABELS | wxSL_HORIZONTAL | wxSL_AUTOTICKS );
    delayVariableLfoPeriodsPerSliceSlider->SetTickFreq ( 10, -1 );
    delayVariableLfoPeriodsPerSliceSlider->SetLineSize ( 1 );
    delayVariableLfoPeriodsPerSliceSlider->SetPageSize ( 1 );

    wxStaticBoxSizer *delayVariableFeedbackSliderSizer, *delayVariableLfoDepthSliderSizer, *delayVariableDistanceSliderSizer,
    *delayVariableGainSliderSizer, *delayVariableLfoPeriodsPerSliceSliderSizer;

    delayVariableDistanceSliderSizer = new wxStaticBoxSizer ( wxVERTICAL, delayVariablePanel, _("distance"));
    delayVariableDistanceSliderSizer->SetMinSize( fxBoxSize );
    delayVariableDistanceSliderSizer->Add ( delayVariableDistanceSlider );
    delayVariableDistanceSliderSizer->AddSpacer ( 5 );
    delayVariableDistanceSliderSizer->Add ( delayVariablePreset1Button );
    delayVariableDistanceSliderSizer->Add ( delayVariablePreset2Button );
    delayVariableDistanceSliderSizer->Add ( delayVariablePreset3Button );
    delayVariableDistanceSliderSizer->Add ( delayVariablePreset4Button );

    delayVariableGainSliderSizer = new wxStaticBoxSizer ( wxVERTICAL, delayVariablePanel, _("gain"));
    delayVariableGainSliderSizer->SetMinSize( fxBoxSize );
    delayVariableGainSliderSizer->Add ( delayVariableGainSlider );

    delayVariableFeedbackSliderSizer = new wxStaticBoxSizer ( wxVERTICAL, delayVariablePanel, _("feedback"));
    delayVariableFeedbackSliderSizer->SetMinSize( fxBoxSize );
    delayVariableFeedbackSliderSizer->Add ( delayVariableFeedbackSlider );

    delayVariableLfoDepthSliderSizer = new wxStaticBoxSizer ( wxVERTICAL, delayVariablePanel, _("lfo depth (%)"));
    delayVariableLfoDepthSliderSizer->SetMinSize( fxBoxSize );
    delayVariableLfoDepthSliderSizer->Add ( delayVariableLfoDepthSlider );

    delayVariableLfoPeriodsPerSliceSliderSizer = new wxStaticBoxSizer ( wxVERTICAL, delayVariablePanel, _("lfo period (%)"));
    delayVariableLfoPeriodsPerSliceSliderSizer->SetMinSize( fxBoxSize );
    delayVariableLfoPeriodsPerSliceSliderSizer->Add ( delayVariableLfoPeriodsPerSliceSlider );

    wxBoxSizer *delayVariableSizer;

    delayVariableSizer = new wxBoxSizer ( wxVERTICAL );
    delayVariableSizer->Add ( delayVariableDistanceSliderSizer );
    delayVariableSizer->Add ( delayVariableGainSliderSizer );
    delayVariableSizer->Add( delayVariableFeedbackSliderSizer );
    delayVariableSizer->Add( delayVariableLfoTypesBox );
    delayVariableSizer->Add( delayVariableLfoDepthSliderSizer );
    delayVariableSizer->Add( delayVariableLfoPeriodsPerSliceSliderSizer );
    delayVariablePanel->SetSizer ( delayVariableSizer );
    delayVariableSizer->Layout();

    wxPanel *delayPhasingPanel;
    delayPhasingPanel = new wxPanel ( effectsTree, wxID_ANY );

    delayPhasingDistanceOffsetSlider = new wxSlider ( delayPhasingPanel, wxID_ANY,
            440, 1, 500, wxDefaultPosition, fxControlSize,
            wxSL_LABELS | wxSL_HORIZONTAL | wxSL_AUTOTICKS );
    delayPhasingDistanceOffsetSlider->SetTickFreq ( 50, -1 );
    delayPhasingDistanceOffsetSlider->SetLineSize ( 1 );
    delayPhasingDistanceOffsetSlider->SetPageSize ( 1 );

    delayPhasingDistanceRangeSlider = new wxSlider ( delayPhasingPanel, wxID_ANY,
            800, 1, 1000, wxDefaultPosition, fxControlSize,
            wxSL_LABELS | wxSL_HORIZONTAL | wxSL_AUTOTICKS );
    delayPhasingDistanceRangeSlider->SetTickFreq ( 100, -1 );
    delayPhasingDistanceRangeSlider->SetLineSize ( 1 );
    delayPhasingDistanceRangeSlider->SetPageSize ( 1 );

    delayPhasingGainSlider = new wxSlider ( delayPhasingPanel, wxID_ANY,
                                            7, 0, 10, wxDefaultPosition, fxControlSize,
                                            wxSL_LABELS | wxSL_HORIZONTAL | wxSL_AUTOTICKS );
    delayPhasingGainSlider->SetTickFreq ( 1, -1 );
    delayPhasingGainSlider->SetLineSize ( 1 );
    delayPhasingGainSlider->SetPageSize ( 2 );

    wxString delayPhasingLfoTypesChoices[] = {_("sine squared"), _("triangle")};
    delayPhasingLfoTypesBox = new wxRadioBox ( delayPhasingPanel, wxID_ANY, _("lfo type"),
            wxDefaultPosition, fxBoxSize, 2, delayLfoTypesChoices, 2, wxRA_SPECIFY_ROWS);
    delayPhasingLfoTypesBox->SetSelection( 0 );
    delayPhasingLfoTypesBox->SetMinSize( fxBoxSize );

    delayPhasingLfoDepthSlider = new wxSlider ( delayPhasingPanel, wxID_ANY,
            1, 0, 100, wxDefaultPosition, fxControlSize,
            wxSL_LABELS | wxSL_HORIZONTAL | wxSL_AUTOTICKS );
    delayPhasingLfoDepthSlider->SetTickFreq ( 10, -1 );
    delayPhasingLfoDepthSlider->SetLineSize ( 1 );
    delayPhasingLfoDepthSlider->SetPageSize ( 10 );

    delayPhasingLfoPeriodsPerSliceSlider = new wxSlider ( delayPhasingPanel, wxID_ANY,
            25, 1, 100, wxDefaultPosition, fxControlSize,
            wxSL_LABELS | wxSL_HORIZONTAL | wxSL_AUTOTICKS );
    delayPhasingLfoPeriodsPerSliceSlider->SetTickFreq ( 10, -1 );
    delayPhasingLfoPeriodsPerSliceSlider->SetLineSize ( 1 );
    delayPhasingLfoPeriodsPerSliceSlider->SetPageSize ( 1 );

    wxStaticBoxSizer *delayPhasingDistanceOffsetSliderSizer, *delayPhasingDistanceRangeSliderSizer, *delayPhasingLfoDepthSliderSizer, *delayPhasingDistanceSliderSizer,
    *delayPhasingGainSliderSizer, *delayPhasingLfoPeriodsPerSliceSliderSizer;

    delayPhasingDistanceOffsetSliderSizer = new wxStaticBoxSizer ( wxVERTICAL, delayPhasingPanel, _("tone"));
    delayPhasingDistanceOffsetSliderSizer->SetMinSize( fxBoxSize );
    delayPhasingDistanceOffsetSliderSizer->Add ( delayPhasingDistanceOffsetSlider );

    delayPhasingDistanceRangeSliderSizer = new wxStaticBoxSizer ( wxVERTICAL, delayPhasingPanel, _("width"));
    delayPhasingDistanceRangeSliderSizer->SetMinSize( fxBoxSize );
    delayPhasingDistanceRangeSliderSizer->Add ( delayPhasingDistanceRangeSlider );

    delayPhasingGainSliderSizer = new wxStaticBoxSizer ( wxVERTICAL, delayPhasingPanel, _("gain"));
    delayPhasingGainSliderSizer->SetMinSize( fxBoxSize );
    delayPhasingGainSliderSizer->Add ( delayPhasingGainSlider );

    delayPhasingLfoDepthSliderSizer = new wxStaticBoxSizer ( wxVERTICAL, delayPhasingPanel, _("lfo depth (%)"));
    delayPhasingLfoDepthSliderSizer->SetMinSize( fxBoxSize );
    delayPhasingLfoDepthSliderSizer->Add ( delayPhasingLfoDepthSlider );

    delayPhasingLfoPeriodsPerSliceSliderSizer = new wxStaticBoxSizer ( wxVERTICAL, delayPhasingPanel, _("lfo period (%)"));
    delayPhasingLfoPeriodsPerSliceSliderSizer->SetMinSize( fxBoxSize );
    delayPhasingLfoPeriodsPerSliceSliderSizer->Add ( delayPhasingLfoPeriodsPerSliceSlider );

    wxBoxSizer *delayPhasingSizer;

    delayPhasingSizer = new wxBoxSizer ( wxVERTICAL );
    delayPhasingSizer->Add ( delayPhasingDistanceOffsetSliderSizer );
    delayPhasingSizer->Add ( delayPhasingDistanceRangeSliderSizer );
    delayPhasingSizer->Add ( delayPhasingGainSliderSizer );
    delayPhasingSizer->Add( delayPhasingLfoTypesBox );
    delayPhasingSizer->Add( delayPhasingLfoDepthSliderSizer );
    delayPhasingSizer->Add( delayPhasingLfoPeriodsPerSliceSliderSizer );
    delayPhasingPanel->SetSizer ( delayPhasingSizer );
    delayPhasingSizer->Layout();



    wxPanel *fftLowPassPanel, *fftHighPassPanel,
    *fftBandPassPanel, *fftBandStopPanel;
    fftLowPassPanel = new wxPanel ( effectsTree, wxID_ANY );
    fftHighPassPanel = new wxPanel ( effectsTree, wxID_ANY );
    fftBandPassPanel = new wxPanel ( effectsTree, wxID_ANY );
    fftBandStopPanel = new wxPanel ( effectsTree, wxID_ANY );

    fftLowPassBandHiSlider = new wxSlider ( fftLowPassPanel, ID_LOWPASS_BAND_HI_SLIDER,
                                            1000, 0, 22050, wxDefaultPosition, fxControlSize,
                                            wxSL_LABELS | wxSL_HORIZONTAL | wxSL_AUTOTICKS );
    fftLowPassBandHiSlider->SetTickFreq ( 2000, -1 );
    fftLowPassBandHiSlider->SetLineSize ( 100 );
    fftLowPassBandHiSlider->SetPageSize ( 1000 );

    fftLowPassSlopeHiSlider = new wxSlider ( fftLowPassPanel, ID_LOWPASS_SLOPE_HI_SLIDER,
            1000, 0, 22050, wxDefaultPosition, fxControlSize,
            wxSL_LABELS | wxSL_HORIZONTAL | wxSL_AUTOTICKS );
    fftLowPassSlopeHiSlider->SetTickFreq ( 2000, -1 );
    fftLowPassSlopeHiSlider->SetLineSize ( 100 );
    fftLowPassSlopeHiSlider->SetPageSize ( 1000 );

    fftHighPassBandLoSlider = new wxSlider ( fftHighPassPanel, ID_HIGHPASS_BAND_LO_SLIDER,
            5500, 0, 22050, wxDefaultPosition, fxControlSize,
            wxSL_LABELS | wxSL_HORIZONTAL | wxSL_AUTOTICKS );
    fftHighPassBandLoSlider->SetTickFreq ( 2000, -1 );
    fftHighPassBandLoSlider->SetLineSize ( 100 );
    fftHighPassBandLoSlider->SetPageSize ( 1000 );

    fftHighPassSlopeLoSlider = new wxSlider ( fftHighPassPanel, ID_HIGHPASS_SLOPE_LO_SLIDER,
            2500, 0, 22050, wxDefaultPosition, fxControlSize,
            wxSL_LABELS | wxSL_HORIZONTAL | wxSL_AUTOTICKS );
    fftHighPassSlopeLoSlider->SetTickFreq ( 2000, -1 );
    fftHighPassSlopeLoSlider->SetLineSize ( 100 );
    fftHighPassSlopeLoSlider->SetPageSize ( 1000 );

    fftBandPassBandLoSlider = new wxSlider ( fftBandPassPanel, ID_BANDPASS_BAND_LO_SLIDER,
            4500, 0, 22050, wxDefaultPosition, fxControlSize,
            wxSL_LABELS | wxSL_HORIZONTAL | wxSL_AUTOTICKS );
    fftBandPassBandLoSlider->SetTickFreq ( 2000, -1 );
    fftBandPassBandLoSlider->SetLineSize ( 100 );
    fftBandPassBandLoSlider->SetPageSize ( 1000 );

    fftBandPassSlopeLoSlider = new wxSlider ( fftBandPassPanel, ID_BANDPASS_SLOPE_LO_SLIDER,
            1500, 0, 22050, wxDefaultPosition, fxControlSize,
            wxSL_LABELS | wxSL_HORIZONTAL | wxSL_AUTOTICKS );
    fftBandPassSlopeLoSlider->SetTickFreq ( 2000, -1 );
    fftBandPassSlopeLoSlider->SetLineSize ( 100 );
    fftBandPassSlopeLoSlider->SetPageSize ( 1000 );

    fftBandPassBandHiSlider = new wxSlider ( fftBandPassPanel, ID_BANDPASS_BAND_HI_SLIDER,
            5500, 0, 22050, wxDefaultPosition, fxControlSize,
            wxSL_LABELS | wxSL_HORIZONTAL | wxSL_AUTOTICKS );
    fftBandPassBandHiSlider->SetTickFreq ( 2000, -1 );
    fftBandPassBandHiSlider->SetLineSize ( 100 );
    fftBandPassBandHiSlider->SetPageSize ( 1000 );

    fftBandPassSlopeHiSlider = new wxSlider ( fftBandPassPanel, ID_BANDPASS_SLOPE_HI_SLIDER,
            1500, 0, 22050, wxDefaultPosition, fxControlSize,
            wxSL_LABELS | wxSL_HORIZONTAL | wxSL_AUTOTICKS );
    fftBandPassSlopeHiSlider->SetTickFreq ( 2000, -1 );
    fftBandPassSlopeHiSlider->SetLineSize ( 100 );
    fftBandPassSlopeHiSlider->SetPageSize ( 1000 );

    fftBandStopBandLoSlider = new wxSlider ( fftBandStopPanel, ID_BANDSTOP_BAND_LO_SLIDER,
            2000, 0, 22050, wxDefaultPosition, fxControlSize,
            wxSL_LABELS | wxSL_HORIZONTAL | wxSL_AUTOTICKS );
    fftBandStopBandLoSlider->SetTickFreq ( 2000, -1 );
    fftBandStopBandLoSlider->SetLineSize ( 100 );
    fftBandStopBandLoSlider->SetPageSize ( 1000 );

    fftBandStopSlopeLoSlider = new wxSlider ( fftBandStopPanel, ID_BANDSTOP_SLOPE_LO_SLIDER,
            1500, 0, 22050, wxDefaultPosition, fxControlSize,
            wxSL_LABELS | wxSL_HORIZONTAL | wxSL_AUTOTICKS );
    fftBandStopSlopeLoSlider->SetTickFreq ( 2000, -1 );
    fftBandStopSlopeLoSlider->SetLineSize ( 100 );
    fftBandStopSlopeLoSlider->SetPageSize ( 1000 );

    fftBandStopBandHiSlider = new wxSlider ( fftBandStopPanel, ID_BANDSTOP_BAND_HI_SLIDER,
            8000, 0, 22050, wxDefaultPosition, fxControlSize,
            wxSL_LABELS | wxSL_HORIZONTAL | wxSL_AUTOTICKS );
    fftBandStopBandHiSlider->SetTickFreq ( 2000, -1 );
    fftBandStopBandHiSlider->SetLineSize ( 100 );
    fftBandStopBandHiSlider->SetPageSize ( 1000 );

    fftBandStopSlopeHiSlider = new wxSlider ( fftBandStopPanel, ID_BANDSTOP_SLOPE_HI_SLIDER,
            1500, 0, 22050, wxDefaultPosition, fxControlSize,
            wxSL_LABELS | wxSL_HORIZONTAL | wxSL_AUTOTICKS );
    fftBandStopSlopeHiSlider->SetTickFreq ( 2000, -1 );
    fftBandStopSlopeHiSlider->SetLineSize ( 100 );
    fftBandStopSlopeHiSlider->SetPageSize ( 1000 );

    lowpassPlot = new FxPlotWindow ( fftLowPassPanel, wxID_ANY, wxPoint ( 0, 0 ), fxPlotSize );
    highpassPlot = new FxPlotWindow ( fftHighPassPanel, wxID_ANY, wxPoint ( 0, 0 ), fxPlotSize );
    bandpassPlot = new FxPlotWindow ( fftBandPassPanel, wxID_ANY, wxPoint ( 0, 0 ), fxPlotSize );
    bandstopPlot = new FxPlotWindow ( fftBandStopPanel, wxID_ANY, wxPoint ( 0, 0 ), fxPlotSize );

    wxStaticBoxSizer
    *fftLowPassBandHiSizer, *fftLowPassSlopeHiSizer, *fftLowPassPlotSizer,
    *fftHighPassBandLoSizer, *fftHighPassSlopeLoSizer, *fftHighPassPlotSizer,
    *fftBandPassBandLoSizer, *fftBandPassSlopeLoSizer, *fftBandPassBandHiSizer, *fftBandPassSlopeHiSizer, *fftBandPassPlotSizer,
    *fftBandStopBandLoSizer, *fftBandStopSlopeLoSizer, *fftBandStopBandHiSizer, *fftBandStopSlopeHiSizer, *fftBandStopPlotSizer;

    fftLowPassPlotSizer = new wxStaticBoxSizer ( wxVERTICAL, fftLowPassPanel, _("preview"));
    fftLowPassPlotSizer->SetMinSize ( fxBoxSize );
    fftLowPassPlotSizer->Add( lowpassPlot );
    fftHighPassPlotSizer = new wxStaticBoxSizer ( wxVERTICAL, fftHighPassPanel, _("preview"));
    fftHighPassPlotSizer->SetMinSize ( fxBoxSize );
    fftHighPassPlotSizer->Add ( highpassPlot );
    fftBandPassPlotSizer = new wxStaticBoxSizer ( wxVERTICAL, fftBandPassPanel, _("preview"));
    fftBandPassPlotSizer->SetMinSize ( fxBoxSize );
    fftBandPassPlotSizer->Add ( bandpassPlot );
    fftBandStopPlotSizer = new wxStaticBoxSizer ( wxVERTICAL, fftBandStopPanel, _("preview"));
    fftBandStopPlotSizer->SetMinSize ( fxBoxSize );
    fftBandStopPlotSizer->Add ( bandstopPlot );

    fftLowPassBandHiSizer = new wxStaticBoxSizer ( wxVERTICAL, fftLowPassPanel, _("cutoff high (hz)"));
    fftLowPassBandHiSizer->SetMinSize( fxBoxSize );
    fftLowPassSlopeHiSizer = new wxStaticBoxSizer ( wxVERTICAL, fftLowPassPanel, _("slope high"));
    fftLowPassSlopeHiSizer->SetMinSize( fxBoxSize );

    fftHighPassBandLoSizer = new wxStaticBoxSizer ( wxVERTICAL, fftHighPassPanel, _("cutoff low (hz)"));
    fftHighPassBandLoSizer->SetMinSize( fxBoxSize );
    fftHighPassSlopeLoSizer = new wxStaticBoxSizer ( wxVERTICAL, fftHighPassPanel, _("slope low"));
    fftHighPassSlopeLoSizer->SetMinSize( fxBoxSize );

    fftBandPassBandLoSizer = new wxStaticBoxSizer ( wxVERTICAL, fftBandPassPanel, _("cutoff low (hz)"));
    fftBandPassBandLoSizer->SetMinSize( fxBoxSize );
    fftBandPassSlopeLoSizer = new wxStaticBoxSizer ( wxVERTICAL, fftBandPassPanel, _("slope low"));
    fftBandPassSlopeLoSizer->SetMinSize( fxBoxSize );
    fftBandPassBandHiSizer = new wxStaticBoxSizer ( wxVERTICAL, fftBandPassPanel, _("cutoff high (hz)"));
    fftBandPassBandHiSizer->SetMinSize( fxBoxSize );
    fftBandPassSlopeHiSizer = new wxStaticBoxSizer ( wxVERTICAL, fftBandPassPanel, _("slope high"));
    fftBandPassSlopeHiSizer->SetMinSize( fxBoxSize );

    fftBandStopBandLoSizer = new wxStaticBoxSizer ( wxVERTICAL, fftBandStopPanel, _("cutoff low (hz)"));
    fftBandStopBandLoSizer->SetMinSize( fxBoxSize );
    fftBandStopSlopeLoSizer = new wxStaticBoxSizer ( wxVERTICAL, fftBandStopPanel, _("slope low"));
    fftBandStopSlopeLoSizer->SetMinSize( fxBoxSize );
    fftBandStopBandHiSizer = new wxStaticBoxSizer ( wxVERTICAL, fftBandStopPanel, _("cutoff high (hz)"));
    fftBandStopBandHiSizer->SetMinSize( fxBoxSize );
    fftBandStopSlopeHiSizer = new wxStaticBoxSizer ( wxVERTICAL, fftBandStopPanel, _("slope high"));
    fftBandStopSlopeHiSizer->SetMinSize( fxBoxSize );

    fftLowPassBandHiSizer->Add ( fftLowPassBandHiSlider );
    fftLowPassSlopeHiSizer->Add ( fftLowPassSlopeHiSlider );

    fftHighPassBandLoSizer->Add ( fftHighPassBandLoSlider );
    fftHighPassSlopeLoSizer->Add ( fftHighPassSlopeLoSlider );

    fftBandPassBandLoSizer->Add ( fftBandPassBandLoSlider );
    fftBandPassSlopeLoSizer->Add ( fftBandPassSlopeLoSlider );
    fftBandPassBandHiSizer->Add ( fftBandPassBandHiSlider );
    fftBandPassSlopeHiSizer->Add ( fftBandPassSlopeHiSlider );

    fftBandStopBandLoSizer->Add ( fftBandStopBandLoSlider );
    fftBandStopSlopeLoSizer->Add ( fftBandStopSlopeLoSlider );
    fftBandStopBandHiSizer->Add ( fftBandStopBandHiSlider );
    fftBandStopSlopeHiSizer->Add ( fftBandStopSlopeHiSlider );

    wxBoxSizer *fftLowPassSizer, *fftHighPassSizer, *fftBandPassSizer, *fftBandStopSizer;

    fftLowPassSizer = new wxBoxSizer ( wxVERTICAL );
    fftLowPassSizer->Add( fftLowPassBandHiSizer );
    fftLowPassSizer->Add ( fftLowPassSlopeHiSizer );
    fftLowPassSizer->Add ( fftLowPassPlotSizer );
    fftLowPassPanel->SetSizer ( fftLowPassSizer );
    fftLowPassSizer->Layout();

    fftHighPassSizer = new wxBoxSizer ( wxVERTICAL );
    fftHighPassSizer->Add ( fftHighPassBandLoSizer );
    fftHighPassSizer->Add ( fftHighPassSlopeLoSizer );
    fftHighPassSizer->Add ( fftHighPassPlotSizer );
    fftHighPassPanel->SetSizer ( fftHighPassSizer );
    fftHighPassSizer->Layout();

    fftBandPassSizer = new wxBoxSizer ( wxVERTICAL );
    fftBandPassSizer->Add ( fftBandPassBandLoSizer );
    fftBandPassSizer->Add ( fftBandPassSlopeLoSizer );
    fftBandPassSizer->Add ( fftBandPassBandHiSizer );
    fftBandPassSizer->Add ( fftBandPassSlopeHiSizer );
    fftBandPassSizer->Add ( fftBandPassPlotSizer );
    fftBandPassPanel->SetSizer ( fftBandPassSizer );
    fftBandPassSizer->Layout();

    fftBandStopSizer = new wxBoxSizer ( wxVERTICAL );
    fftBandStopSizer->Add ( fftBandStopBandLoSizer );
    fftBandStopSizer->Add ( fftBandStopSlopeLoSizer );
    fftBandStopSizer->Add ( fftBandStopBandHiSizer );
    fftBandStopSizer->Add ( fftBandStopSlopeHiSizer );
    fftBandStopSizer->Add ( fftBandStopPlotSizer );
    fftBandStopPanel->SetSizer ( fftBandStopSizer );
    fftBandStopSizer->Layout();

    wxPanel *bitCrushPanel;
    bitCrushPanel = new wxPanel ( effectsTree, wxID_ANY );

    wxString bitCrushTypesChoices[] = { _("4 bit"), _("6 bit"), _("8 bit"), _("10 bit") };
    bitCrushTypesBox = new wxRadioBox ( bitCrushPanel, wxID_ANY, _("type"),
                                        wxDefaultPosition, fxBoxSize, 4, bitCrushTypesChoices, 4, wxRA_SPECIFY_ROWS);
    wxBoxSizer *bitCrushSizer;
    bitCrushSizer = new wxBoxSizer ( wxVERTICAL );
    bitCrushSizer->Add ( bitCrushTypesBox );
    bitCrushSizer->Layout();

    wxPanel *downSamplePanel;
    downSamplePanel = new wxPanel ( effectsTree, wxID_ANY );

    wxString downSampleTypesChoices[] = { _("44,100 Hz"), _("22,050 Hz"), _("11,025 Hz"), _("5,513 Hz"), _("2,756 Hz"), _("1,378 Hz") };
    downSampleTypesBox = new wxRadioBox ( downSamplePanel, wxID_ANY, _("type"),
                                          wxDefaultPosition, fxBoxSize, 6, downSampleTypesChoices, 6, wxRA_SPECIFY_ROWS);
    downSampleTypesBox->SetSelection ( 2 );
    wxBoxSizer *downSampleSizer;
    downSampleSizer = new wxBoxSizer ( wxVERTICAL );
    downSampleSizer->Add ( downSampleTypesBox );
    downSampleSizer->Layout();

    wxPanel *distortPanel;
    distortPanel = new wxPanel ( effectsTree, wxID_ANY );

    distortCutoffSlider = new wxSlider ( distortPanel, wxID_ANY,
                                         4000, 1000, 6000, wxDefaultPosition, fxControlSize,
                                         wxSL_LABELS | wxSL_HORIZONTAL | wxSL_AUTOTICKS );
    distortCutoffSlider->SetTickFreq ( 1000, -1 );
    distortCutoffSlider->SetLineSize ( 50 );
    distortCutoffSlider->SetPageSize ( 500 );

    distortRangeSlider = new wxSlider ( distortPanel, wxID_ANY,
                                        500, 0, 1000, wxDefaultPosition, fxControlSize,
                                        wxSL_LABELS | wxSL_HORIZONTAL | wxSL_AUTOTICKS );
    distortRangeSlider->SetTickFreq ( 100, -1 );
    distortRangeSlider->SetLineSize ( 50 );
    distortRangeSlider->SetPageSize ( 100 );

    distortAmplificationFactorSlider = new wxSlider ( distortPanel, wxID_ANY,
            2, 1, 5,
            wxDefaultPosition, fxControlSize,
            wxSL_LABELS | wxSL_HORIZONTAL | wxSL_AUTOTICKS );
    distortAmplificationFactorSlider->SetTickFreq ( 1, -1 );

    wxBoxSizer *distortSizer;
    distortSizer = new wxBoxSizer ( wxVERTICAL );

    wxStaticBoxSizer *distortCutoffSizer, *distortRangeSizer, *distortAmplificationSizer;
    distortCutoffSizer = new wxStaticBoxSizer ( wxVERTICAL, distortPanel, _("cutoff"));
    distortRangeSizer = new wxStaticBoxSizer ( wxVERTICAL, distortPanel, _("range"));
    distortAmplificationSizer = new wxStaticBoxSizer ( wxVERTICAL, distortPanel, _("gain"));

    distortCutoffSizer->Add ( distortCutoffSlider );
    distortCutoffSizer->SetMinSize ( fxBoxSize );
    distortSizer->Add ( distortCutoffSizer );

    distortRangeSizer->Add ( distortRangeSlider );
    distortRangeSizer->SetMinSize ( fxBoxSize );
    distortSizer->Add ( distortRangeSizer );

    distortAmplificationSizer->Add ( distortAmplificationFactorSlider );
    distortAmplificationSizer->SetMinSize ( fxBoxSize );
    distortSizer->Add ( distortAmplificationSizer );

    distortPanel->SetSizer ( distortSizer );
    distortSizer->Layout();

    wxPanel *fadeInPanel, *fadeOutPanel;
    fadeInPanel = new wxPanel ( effectsTree, wxID_ANY );
    fadeOutPanel = new wxPanel ( effectsTree, wxID_ANY );

    wxString fadeTypesChoices[] = { _("log10"), _("log5"), _("log3"), _("log1.5") };
    fadeOutTypesBox = new wxRadioBox ( fadeOutPanel, wxID_ANY, _("type"),
                                       wxDefaultPosition, fxBoxSize, 4, fadeTypesChoices, 4, wxRA_SPECIFY_ROWS);
    fadeInTypesBox = new wxRadioBox ( fadeInPanel, wxID_ANY, _("type"),
                                      wxDefaultPosition, fxBoxSize, 4, fadeTypesChoices, 4, wxRA_SPECIFY_ROWS);

    fadeInRangeSlider = new wxSlider ( fadeInPanel, wxID_ANY,
                                       3000, 0, 8000, wxDefaultPosition, fxControlSize,
                                       wxSL_LABELS | wxSL_HORIZONTAL | wxSL_AUTOTICKS );
    fadeInRangeSlider->SetTickFreq ( 1000, -1 );
    fadeInRangeSlider->SetLineSize ( 50 );
    fadeInRangeSlider->SetPageSize ( 100 );

    fadeOutRangeSlider = new wxSlider ( fadeOutPanel, wxID_ANY,
                                        3000, 0, 8000, wxDefaultPosition, fxControlSize,
                                        wxSL_LABELS | wxSL_HORIZONTAL | wxSL_AUTOTICKS );
    fadeOutRangeSlider->SetTickFreq ( 1000, -1 );
    fadeOutRangeSlider->SetLineSize ( 50 );
    fadeOutRangeSlider->SetPageSize ( 100 );

    wxBoxSizer *fadeInSizer, *fadeOutSizer;
    fadeInSizer = new wxBoxSizer ( wxVERTICAL );
    fadeOutSizer = new wxBoxSizer ( wxVERTICAL );

    wxStaticBoxSizer *fadeInRangeSizer, *fadeOutRangeSizer;
    fadeInRangeSizer = new wxStaticBoxSizer ( wxVERTICAL, fadeInPanel, _("range"));
    fadeOutRangeSizer = new wxStaticBoxSizer ( wxVERTICAL, fadeOutPanel, _("range"));

    fadeInRangeSizer->SetMinSize ( fxBoxSize );
    fadeInRangeSizer->Add ( fadeInRangeSlider );
    fadeInTypesBox->SetMinSize ( fxBoxSize );
    fadeOutRangeSizer->SetMinSize ( fxBoxSize );
    fadeOutRangeSizer->Add ( fadeOutRangeSlider );
    fadeOutTypesBox->SetMinSize ( fxBoxSize );

    fadeInSizer->Add ( fadeInTypesBox );
    fadeInSizer->Add ( fadeInRangeSizer );
    fadeInPanel->SetSizer ( fadeInSizer ) ;
    fadeOutSizer->Add ( fadeOutTypesBox );
    fadeOutSizer->Add ( fadeOutRangeSizer );
    fadeOutPanel->SetSizer ( fadeOutSizer ) ;

    fadeInSizer->Layout();
    fadeOutSizer->Layout();

    wxPanel *retriggerPanel;
    retriggerPanel = new wxPanel ( effectsTree, wxID_ANY );

    retriggerCountSlider = new wxSlider ( retriggerPanel, wxID_ANY,
                                          2, 1, 20,
                                          wxDefaultPosition, fxControlSize,
                                          wxSL_LABELS | wxSL_HORIZONTAL | wxSL_AUTOTICKS );
    retriggerCountSlider->SetTickFreq ( 1, -1 );
    retriggerCountSlider->SetLineSize ( 1 );
    retriggerCountSlider->SetPageSize ( 2 );

    wxBoxSizer *retriggerSizer;
    retriggerSizer = new wxBoxSizer ( wxVERTICAL );

    wxStaticBoxSizer *retriggerSliderSizer;
    retriggerSliderSizer = new wxStaticBoxSizer ( wxVERTICAL, retriggerPanel, _("trigger"));

    retriggerSliderSizer->Add ( retriggerCountSlider );
    retriggerSliderSizer->SetMinSize ( fxBoxSize );
    retriggerSizer->Add ( retriggerSliderSizer );

    retriggerPanel->SetSizer ( retriggerSizer );
    retriggerSizer->Layout();


    wxPanel *tremoloPanel;
    tremoloPanel = new wxPanel ( effectsTree, wxID_ANY );

    wxString tremoloTypesChoices[] = {_("sine squared"), _("sine"), _("sawtooth"), _("square"), _("triangle")};
    tremoloTypesBox = new wxRadioBox ( tremoloPanel, ID_TREMOLO_TYPES_BOX, _("type"),
                                       wxDefaultPosition, fxBoxSize, 5, tremoloTypesChoices, 5, wxRA_SPECIFY_ROWS);
    tremoloTypesBox->SetSelection( 3 );
    tremoloTypesBox->SetMinSize( fxBoxSize );

    wxString tremoloStereoModeChoices[] = {_("synchronous"), _("ping pong"), _("left delayed"), _("right delayed")};
    tremoloStereoModeBox = new wxRadioBox ( tremoloPanel, ID_TREMOLO_STEREO_MODE_BOX, _("pan"),
                                            wxDefaultPosition, fxBoxSize, 4, tremoloStereoModeChoices, 4, wxRA_SPECIFY_ROWS);
    tremoloStereoModeBox->SetSelection( 0 );
    tremoloStereoModeBox->SetMinSize( fxBoxSize );

    tremoloDepthSlider = new wxSlider ( tremoloPanel, ID_TREMOLO_DEPTH_SLIDER,
                                        100, 0, 100, wxDefaultPosition, fxControlSize,
                                        wxSL_LABELS | wxSL_HORIZONTAL | wxSL_AUTOTICKS );
    tremoloDepthSlider->SetTickFreq ( 10, -1 );
    tremoloDepthSlider->SetLineSize ( 5 );
    tremoloDepthSlider->SetPageSize ( 10 );

    tremoloPeriodsPerSliceSlider = new wxSlider ( tremoloPanel, ID_TREMOLO_PPS_SLIDER,
            2, 1, 40, wxDefaultPosition, fxControlSize,
            wxSL_LABELS | wxSL_HORIZONTAL | wxSL_AUTOTICKS );
    tremoloPeriodsPerSliceSlider->SetTickFreq ( 10, -1 );
    tremoloPeriodsPerSliceSlider->SetLineSize ( 2 );
    tremoloPeriodsPerSliceSlider->SetPageSize ( 5 );

    wxBoxSizer *tremoloSizer;
    tremoloSizer = new wxBoxSizer ( wxVERTICAL );

    tremoloPlot = new FxPlotWindow ( tremoloPanel, wxID_ANY, wxPoint ( 0, 0 ), fxPlotSize );
    wxStaticBoxSizer *tremoloDepthSizer, *tremoloPeriodsPerSliceSizer, *tremoloPlotSizer;
    tremoloDepthSizer = new wxStaticBoxSizer ( wxVERTICAL, tremoloPanel, _("depth (%)"));
    tremoloPeriodsPerSliceSizer = new wxStaticBoxSizer ( wxVERTICAL, tremoloPanel, _("periods per slice"));
    tremoloPlotSizer = new wxStaticBoxSizer ( wxVERTICAL, tremoloPanel, _("preview"));

    tremoloDepthSizer->Add ( tremoloDepthSlider );
    tremoloDepthSizer->SetMinSize( fxBoxSize );
    tremoloPeriodsPerSliceSizer->Add ( tremoloPeriodsPerSliceSlider );
    tremoloPeriodsPerSliceSizer->SetMinSize( fxBoxSize );

    tremoloPlotSizer->Add ( tremoloPlot );
    tremoloPlotSizer->SetMinSize ( fxBoxSize );

    tremoloSizer->Add ( tremoloTypesBox );
    tremoloSizer->Add ( tremoloStereoModeBox );
    tremoloSizer->Add ( tremoloDepthSizer );
    tremoloSizer->Add ( tremoloPeriodsPerSliceSizer );
    tremoloSizer->Add ( tremoloPlotSizer );
    tremoloSizer->Layout();

    effectsTree->AddPage( new wxPanel ( effectsTree, wxID_ANY ), _("all") );
    effectsTree->AddSubPage( showFxPanel, _("show") );
    effectsTree->AddPage( new wxPanel ( effectsTree, wxID_ANY ), _("special"));
    effectsTree->AddSubPage( distortPanel, _("distortion"));
    effectsTree->AddSubPage( bitCrushPanel, _("bitcrusher"));
    effectsTree->AddSubPage( downSamplePanel, _("downsample"));
    effectsTree->AddSubPage( retriggerPanel, _("retrigger"));
    effectsTree->AddPage( new wxPanel ( effectsTree, wxID_ANY ), _("filters"));
    effectsTree->AddSubPage( fftLowPassPanel, _("lowpass"));
    effectsTree->AddSubPage( fftHighPassPanel, _("highpass"));
    effectsTree->AddSubPage( fftBandPassPanel, _("bandpass"));
    effectsTree->AddSubPage( fftBandStopPanel, _("bandstop"));
    effectsTree->AddPage( new wxPanel ( effectsTree, wxID_ANY ), _("delay"));
    effectsTree->AddSubPage( delayMultiTapPanel, _("multitap"));
    effectsTree->AddSubPage( delayVariablePanel, _("flanging"));
    effectsTree->AddSubPage( delayPhasingPanel, _("phasing"));
    effectsTree->AddPage( new wxPanel ( effectsTree, wxID_ANY ), _("amplitude"));
    effectsTree->AddSubPage( tremoloPanel, _("tremolo"));
    effectsTree->AddSubPage( fadeInPanel, _("fade in"));
    effectsTree->AddSubPage( fadeOutPanel, _("fade out"));
    effectsTree->AddPage( new wxPanel ( effectsTree, wxID_ANY ), _("time/pitch"));
    effectsTree->AddSubPage( new wxPanel ( effectsTree, wxID_ANY ), _("tapestop"));
    effectsTree->AddSubPage( new wxPanel ( effectsTree, wxID_ANY ), _("reverse"));
    effectsTree->GetTreeCtrl()->ExpandAll();
    effectsTree->Layout();

    generalTree->AddPage ( new wxPanel ( generalTree, wxID_ANY ), _("finishing"));
    generalTree->AddSubPage ( compressPanel, _("compress") );
    generalTree->AddSubPage ( gatePanel, _("gate") );
    generalTree->AddSubPage ( loudnessPanel, _("loudness") );
    generalTree->AddSubPage ( normalizePanel, _("normalize") );
    generalTree->AddSubPage ( pitchAdjustmentPanel, _("pitch") );
    generalTree->AddPage ( new wxPanel ( generalTree, wxID_ANY ), _("master filter"));
    generalTree->AddSubPage ( masterFilterPanel, _("filter") );
    generalTree->AddSubPage ( masterFilterResPanel, _("q") );
    generalTree->AddSubPage ( masterFilterLfoPanel, _("lfo") );
    generalTree->AddSubPage ( masterFilterModPanel, _("sweep") );
    generalTree->AddPage ( new wxPanel ( generalTree, wxID_ANY ), _("shuffle"));
    generalTree->AddSubPage ( keyMapperPanel, _("permute") );
    generalTree->GetTreeCtrl()->ExpandAll();
    generalTree->Layout();

    wxPanel *effectsChainPanel = new wxPanel ( this, wxID_ANY );
    std::vector<wxString> v;
    effectsGrid->getEffectsVector ( v );
    int width = //min 15
#ifndef __WXMSW__
        ( int ) ( bookWidth / v.size() );//std::max ( std::min ( ( int ) ( 0.97 * ( bookWidth / v.size())), 18 ), 15 )
#else
        ( int ) ( bookWidth / v.size() );//std::max ( std::min ( ( int ) ( ( bookWidth / v.size())), 17 ), 15 )
#endif
    ;
    effectsChainGrid = new ChainGrid ( effectsChainPanel, wxID_ANY, v, effectsGrid->GetNumberRows(),
#ifndef __WXMSW__
                                       26
#else
                                       24
#endif
                                       , width );
    width = 0.98 * bookWidth;

    effectsChainDefaultButton = new wxButton ( effectsChainPanel, ID_EFFECTSCHAIN_DEFAULT, _T ( "default"), wxDefaultPosition, wxSize ( width, -1 ) );
    effectsChainClearAllButton = new wxButton ( effectsChainPanel, ID_EFFECTSCHAIN_CLEARALL, _T ( "clear all"), wxDefaultPosition, wxSize ( width, -1 ) );
    wxBoxSizer *effectsChainSizer = new wxBoxSizer ( wxVERTICAL );
    effectsChainSizer->AddSpacer ( 5 );
    effectsChainSizer->Add ( effectsChainGrid );
    effectsChainSizer->AddSpacer ( 10 );
    effectsChainSizer->Add ( effectsChainDefaultButton );
    effectsChainSizer->Add ( effectsChainClearAllButton );
    effectsChainPanel->SetSizer ( effectsChainSizer );
    effectsChainSizer->Layout();

    wxPanel *defaultSamplePanel;
    defaultSamplePanel = new wxPanel ( this, wxID_ANY );
    wxString defaultTypesChoices[] = { _("amen break"), _("piano"), _("tr-808"), _("synth"), _("blank") };
    int defaultChoicesNo = 5;
    defaultTypesBox = new wxRadioBox ( defaultSamplePanel, ID_DEFAULT_SAMPLE_BOX, _(""),
                                       wxDefaultPosition, wxSize ( ( int ) ( width ), -1 ),
                                       defaultChoicesNo, defaultTypesChoices, defaultChoicesNo, wxRA_SPECIFY_ROWS);
    wxBoxSizer *defaultSampleSizer;
    defaultSampleSizer = new wxBoxSizer ( wxVERTICAL );
    defaultSampleSizer->Add ( defaultTypesBox );
    defaultSamplePanel->SetSizer ( defaultSampleSizer );
    defaultSampleSizer->Layout();

    wxPanel *playbackPanel;
    playbackPanel = new wxPanel ( this, wxID_ANY );
    playbackSlider = new wxSlider ( playbackPanel, ID_PLAYBACK_COMPENSATION_SLIDER,
                                    50, 0, 200, wxDefaultPosition, wxSize ( ( int ) (
#ifndef __WXMSW__
    0.975
#else
    0.95
#endif
                                    * width ), -1 ),
                                    wxSL_LABELS | wxSL_HORIZONTAL | wxSL_AUTOTICKS );
    playbackSlider->SetTickFreq ( 20, -1 );
    playbackSlider->SetLineSize ( 10 );
    playbackSlider->SetPageSize ( 20 );
    playbackCheckbox = new wxCheckBox ( playbackPanel, ID_PLAYBACK_COMPENSATION_CHECKBOX, _("enable compensation") );
    playbackCheckbox->SetValue( true );
    wxStaticBoxSizer *playbackSliderSizer;
    playbackSliderSizer = new wxStaticBoxSizer ( wxVERTICAL, playbackPanel, _("gap compensation (ms)"));
    playbackSliderSizer->SetMinSize ( wxSize ( ( int ) ( width ), -1 ) );
    playbackSliderSizer->Add ( playbackSlider );
    wxBoxSizer *playbackSizer;
    playbackSizer = new wxBoxSizer ( wxVERTICAL );
    playbackSizer->Add ( playbackSliderSizer );
    playbackSizer->AddSpacer ( 10 );
    playbackSizer->Add ( playbackCheckbox );
    playbackSizer->AddSpacer ( 10 );
    playbackPanel->SetSizer ( playbackSizer );
    playbackSizer->Layout();

    AddPage ( generalTree, _("master"), false );
    AddPage ( effectsTree, _("effects"), false );
    AddPage ( effectsChainPanel, _("chain"), false );
    AddPage ( defaultSamplePanel, _("presets"), false );
    AddPage ( playbackPanel, _("playback"), false );

    updateShowFxRows ();

    Layout();
}

void SettingsBook::OnButton( wxCommandEvent& event )
{
    size_t rowNo, i;
    int showFxId;

    smasherFrame *frame = ( smasherFrame * ) parent;

    switch ( event.GetId() )
    {
    case ID_DELAY_MULTITAP_PRESET1:
        delayMultiTapDistanceSlider->SetValue ( 1 );
        break;
    case ID_DELAY_MULTITAP_PRESET2:
        delayMultiTapDistanceSlider->SetValue ( 2 );
        break;
    case ID_DELAY_MULTITAP_PRESET3:
        delayMultiTapDistanceSlider->SetValue ( 13 );
        break;
    case ID_DELAY_MULTITAP_PRESET4:
        delayMultiTapDistanceSlider->SetValue ( 25 );
        break;
    case ID_DELAY_MULTITAP_PRESET5:
        delayMultiTapDistanceSlider->SetValue ( 33 );
        break;
    case ID_DELAY_MULTITAP_PRESET6:
        delayMultiTapDistanceSlider->SetValue ( 50 );
        break;
    case ID_DELAY_VARIABLE_PRESET1:
        delayVariableDistanceSlider->SetValue ( 32 );
        break;
    case ID_DELAY_VARIABLE_PRESET2:
        delayVariableDistanceSlider->SetValue ( 64 );
        break;
    case ID_DELAY_VARIABLE_PRESET3:
        delayVariableDistanceSlider->SetValue ( 128 );
        break;
    case ID_DELAY_VARIABLE_PRESET4:
        delayVariableDistanceSlider->SetValue ( 256 );
        break;

    case ID_SHOW_FX_ALL:
        rowNo = effectsGrid->GetNumberRows();
        showFxId = ID_EFFECTS_START;
        for ( i = 0; i < rowNo; i++ )
        {
            showFxId = ID_EFFECTS_START + i;
            wxToggleButton *currentToggleButton = ( wxToggleButton* ) FindWindowById ( showFxId );
            currentToggleButton->SetValue ( 1 );
        }
        updateShowFxRows ();
        break;
    case ID_SHOW_FX_NONE:
        rowNo = effectsGrid->GetNumberRows();
        showFxId = ID_EFFECTS_START;
        for ( i = 0; i < rowNo; i++ )
        {
            showFxId = ID_EFFECTS_START + i;
            wxToggleButton *currentToggleButton = ( wxToggleButton* ) FindWindowById ( showFxId );
            currentToggleButton->SetValue ( 0 );
        }
        updateShowFxRows ();
        break;
    case ID_EFFECTSCHAIN_CLEARALL:
        frame->DoClearOrder ( effectsChainGrid );
        break;
    case ID_EFFECTSCHAIN_DEFAULT:
        frame->DoSliceOrder1ToN ( effectsChainGrid );
        break;
    }
    event.Skip();
}

void SettingsBook::updateShowFxRows ()
{
    int rowNo, showFxId, i;
    rowNo = effectsGrid->GetNumberRows();
    showFxId = ID_EFFECTS_START;
    for ( i = 0; i < rowNo; i++ )
    {
        showFxId = ID_EFFECTS_START + i;
        wxToggleButton *currentToggleButton = ( wxToggleButton * ) FindWindowById ( showFxId );
        effectsGrid->showRow ( effectsGrid->GetRowLabelValue( i ), currentToggleButton->GetValue() );
    }
}

void SettingsBook::refreshPlotAreas()
{
    tremoloPlot->Refresh();
    lowpassPlot->Refresh();
    highpassPlot->Refresh();
    bandpassPlot->Refresh();
    bandstopPlot->Refresh();
    masterFilterPlot->Refresh();
    masterFilterLfoPlot->Refresh();
}

void SettingsBook::initPlotAreas()
{
    refreshPlotAreas();
    tremoloPlotUpdateRequired = true;
    lowpassPlotUpdateRequired = true;
    highpassPlotUpdateRequired = true;
    bandpassPlotUpdateRequired = true;
    bandstopPlotUpdateRequired = true;
    loudnessPlotUpdateRequired = true;
    masterFilterPlotUpdateRequired = true;
    masterFilterModPlotUpdateRequired = true;
    masterFilterLfoPlotUpdateRequired = true;
}

void SettingsBook::OnTreeChanged( wxTreebookEvent& event )
{
    initPlotAreas();
    event.Skip();
}

void SettingsBook::OnBookChanged( wxAuiNotebookEvent& event )
{
    event.Skip();
}

void SettingsBook::OnSize ( wxSizeEvent& event )
{
    initPlotAreas();
    event.Skip();
}

void SettingsBook::OnIdle ( wxIdleEvent& event )
{
    if ( tremoloPlotUpdateRequired )
    {
        updateTremoloPlotArea();
        tremoloPlotUpdateRequired = false;
    }
    if ( lowpassPlotUpdateRequired )
    {
        updateLowpassPlotArea();
        lowpassPlotUpdateRequired = false;
    }
    if ( highpassPlotUpdateRequired )
    {
        updateHighpassPlotArea();
        highpassPlotUpdateRequired = false;
    }
    if ( bandpassPlotUpdateRequired )
    {
        updateBandpassPlotArea();
        bandpassPlotUpdateRequired = false;
    }
    if ( bandstopPlotUpdateRequired )
    {
        updateBandstopPlotArea();
        bandstopPlotUpdateRequired = false;
    }
    if ( loudnessPlotUpdateRequired )
    {
        updateLoudnessPlotArea();
        loudnessPlotUpdateRequired = false;
    }
    if ( masterFilterPlotUpdateRequired )
    {
        updateMasterFilterPlotArea();
        masterFilterPlotUpdateRequired = false;
    }
    if ( masterFilterModPlotUpdateRequired )
    {
        updateMasterFilterModPlotArea();
        masterFilterModPlotUpdateRequired = false;
    }
    if ( masterFilterLfoPlotUpdateRequired )
    {
        updateMasterFilterLfoPlotArea();
        masterFilterLfoPlotUpdateRequired = false;
    }
}

void SettingsBook::OnRadioBoxChanged ( wxCommandEvent& event )
{
    smasherFrame *frame = (smasherFrame *) parent;
    switch ( event.GetId() )
    {
    case ID_TREMOLO_TYPES_BOX:
        tremoloPlotUpdateRequired = true;
        tremoloPlot->Refresh();
        break;
    case ID_TREMOLO_STEREO_MODE_BOX:
        tremoloPlotUpdateRequired = true;
        tremoloPlot->Refresh();
        break;
    case ID_MASTER_FILTER_TYPES_BOX:
        masterFilterPlotUpdateRequired = true;
        masterFilterPlot->Refresh();
        break;
    case ID_MASTER_FILTER_MOD_TYPES_BOX:
        masterFilterModPlotUpdateRequired = true;
        masterFilterModPlot->Refresh();
        break;
    case ID_MASTER_FILTER_LFO_TYPES_BOX:
        masterFilterLfoPlotUpdateRequired = true;
        masterFilterLfoPlot->Refresh();
        break;
    case ID_DEFAULT_SAMPLE_BOX:
        frame->loadState ( frame->getDefaultStates()->at ( event.GetSelection() ), true );
        break;
    default:
        break;
    }
}

void SettingsBook::OnCheckboxChanged ( wxCommandEvent& event )
{
    smasherFrame *frame = (smasherFrame *) parent;
    int eventId = event.GetId();
    if ( eventId == ID_PLAYBACK_COMPENSATION_CHECKBOX )
        frame->setLoopLatency ( ( playbackCheckbox->IsChecked() ) ? playbackSlider->GetValue() : 0);

    event.Skip ();
}

void SettingsBook::OnToggleButtonChanged ( wxCommandEvent& event )
{
    int eventId = event.GetId();
    effectsGrid->showRow ( effectsGrid->GetRowLabelValue( eventId - ID_EFFECTS_START ), event.IsChecked() );
    event.Skip();
}

void SettingsBook::OnSliderChanged ( wxScrollEvent& event )
{
    smasherFrame *frame = (smasherFrame *) parent;
    switch ( event.GetId() )
    {
    case ID_TREMOLO_DEPTH_SLIDER:
        tremoloPlotUpdateRequired = true;
        tremoloPlot->Refresh();
        break;

    case ID_TREMOLO_PPS_SLIDER:
        tremoloPlotUpdateRequired = true;
        tremoloPlot->Refresh();
        break;

    case ID_LOWPASS_BAND_HI_SLIDER:
        lowpassPlotUpdateRequired = true;
        lowpassPlot->Refresh();
        break;
    case ID_LOWPASS_SLOPE_HI_SLIDER:
        lowpassPlotUpdateRequired = true;
        lowpassPlot->Refresh();
        break;

    case ID_HIGHPASS_BAND_LO_SLIDER:
        highpassPlotUpdateRequired = true;
        highpassPlot->Refresh();
        break;
    case ID_HIGHPASS_SLOPE_LO_SLIDER:
        highpassPlotUpdateRequired = true;
        highpassPlot->Refresh();
        break;

    case ID_BANDPASS_BAND_LO_SLIDER:
        bandpassPlotUpdateRequired = true;
        bandpassPlot->Refresh();
        break;
    case ID_BANDPASS_SLOPE_LO_SLIDER:
        bandpassPlotUpdateRequired = true;
        bandpassPlot->Refresh();
        break;
    case ID_BANDPASS_BAND_HI_SLIDER:
        bandpassPlotUpdateRequired = true;
        bandpassPlot->Refresh();
        break;
    case ID_BANDPASS_SLOPE_HI_SLIDER:
        bandpassPlotUpdateRequired = true;
        bandpassPlot->Refresh();
        break;

    case ID_BANDSTOP_BAND_LO_SLIDER:
        bandstopPlotUpdateRequired = true;
        bandstopPlot->Refresh();
        break;
    case ID_BANDSTOP_SLOPE_LO_SLIDER:
        bandstopPlotUpdateRequired = true;
        bandstopPlot->Refresh();
        break;
    case ID_BANDSTOP_BAND_HI_SLIDER:
        bandstopPlotUpdateRequired = true;
        bandstopPlot->Refresh();
        break;
    case ID_BANDSTOP_SLOPE_HI_SLIDER:
        bandstopPlotUpdateRequired = true;
        bandstopPlot->Refresh();
        break;

    case ID_LOUDNESS_BAND_LO_SLIDER:
        loudnessPlotUpdateRequired = true;
        loudnessPlot->Refresh();
        break;
    case ID_LOUDNESS_SLOPE_LO_SLIDER:
        loudnessPlotUpdateRequired = true;
        loudnessPlot->Refresh();
        break;
    case ID_LOUDNESS_BAND_HI_SLIDER:
        loudnessPlotUpdateRequired = true;
        loudnessPlot->Refresh();
        break;
    case ID_LOUDNESS_SLOPE_HI_SLIDER:
        loudnessPlotUpdateRequired = true;
        loudnessPlot->Refresh();
        break;
    case ID_MASTER_FILTER_BAND_LO_SLIDER:
        masterFilterPlotUpdateRequired = true;
        masterFilterPlot->Refresh();
        break;
    case ID_MASTER_FILTER_SLOPE_LO_SLIDER:
        masterFilterPlotUpdateRequired = true;
        masterFilterPlot->Refresh();
        break;
    case ID_MASTER_FILTER_BAND_HI_SLIDER:
        masterFilterPlotUpdateRequired = true;
        masterFilterPlot->Refresh();
        break;
    case ID_MASTER_FILTER_SLOPE_HI_SLIDER:
        masterFilterPlotUpdateRequired = true;
        masterFilterPlot->Refresh();
        break;
    case ID_MASTER_FILTER_MOD_DEPTH_SLIDER:
        masterFilterModPlotUpdateRequired = true;
        masterFilterModPlot->Refresh();
        break;
    case ID_MASTER_FILTER_LFO_DEPTH_SLIDER:
        masterFilterLfoPlotUpdateRequired = true;
        masterFilterLfoPlot->Refresh();
        break;
    case ID_MASTER_FILTER_LFO_PPS_SLIDER:
        masterFilterLfoPlotUpdateRequired = true;
        masterFilterLfoPlot->Refresh();
        break;
    case ID_PLAYBACK_COMPENSATION_SLIDER:
        if ( playbackCheckbox->IsChecked() )
            frame->setLoopLatency ( playbackSlider->GetValue() );
        break;
    default:
        break;
    }
    event.Skip();
}

void SettingsBook::updateTremoloPlotArea ()
{
    if ( !tremoloPlot )
    {
        std::cout << "!tremoloplot" << std::endl;
        return;
    }

    int tremoloType = tremoloTypesBox->GetSelection();
    double tremoloDepth = 0.01 * ( double ) ( tremoloDepthSlider->GetValue() );
    double tremoloPps = ( double ) ( tremoloPeriodsPerSliceSlider->GetValue() );
    double ( *lfo ) ( double );
    switch ( tremoloType )
    {
    case TREMOLO_SAWTOOTH:
        lfo = aaSawTooth;
        break;
    case TREMOLO_SQUARE:
        lfo = aaSquare;
        break;
    case TREMOLO_TRIANGLE:
        lfo = aaTriangle;
        break;
    case TREMOLO_SIN:
        lfo = sinPlus;
        break;
    case TREMOLO_SIN_POW2:
    default:
        lfo = sinPow2;
    }

    tremoloData.clear();

    double increment = ( 2.0 * PI ) / ( double ) fxPlotSize.GetWidth();
    for ( double d = 0.0; d < 2.0 * PI; d += increment )
    {
        tremoloData.push_back (
            ( 1.0 - tremoloDepth ) + tremoloDepth * lfo ( tremoloPps * d ));
    }

    tremoloPlot->set ( &tremoloData );

    if ( tremoloData.empty() )
    {
        std::cout << "tremoloData.empty()" << std::endl;
        return;
    }

}

void SettingsBook::updateLowpassPlotArea ()
{
    if ( !lowpassPlot )
    {
        std::cout << "!lowpassPlot" << std::endl;
        return;
    }

    double bh =  ( double ) fftLowPassBandHiSlider->GetValue() * 0.5 / 22050;
    double sh = ( double ) fftLowPassSlopeHiSlider->GetValue() * 0.5 / 22050;

    lowpassData.clear();

    double increment = 0.5 / (double) fxPlotSize.GetWidth();
    for ( double d = 0.0; d < 0.5; d += increment )
    {
        lowpassData.push_back (
            fftLowpassFilter (
                d, 0.0, 0.0,
                bh, sh) );
    }

    lowpassPlot->set ( &lowpassData );

    if ( lowpassData.empty() )
    {
        std::cout << "lowpassData.empty()" << std::endl;
        return;
    }

}


void SettingsBook::updateHighpassPlotArea ()
{
    if ( !highpassPlot )
    {
        std::cout << "!highpassPlot" << std::endl;
        return;
    }

    double bl =  ( double ) fftHighPassBandLoSlider->GetValue() * 0.5 / 22050;
    double sl = ( double ) fftHighPassSlopeLoSlider->GetValue() * 0.5 / 22050;

    highpassData.clear();

    double increment = 0.5 / ( double ) fxPlotSize.GetWidth();
    for ( double d = 0.0; d < 0.5; d += increment )
    {
        highpassData.push_back (
            fftHighpassFilter (
                d, bl, sl,
                0.0, 0.0 ) );
    }

    highpassPlot->set ( &highpassData );

    if ( highpassData.empty() )
    {
        std::cout << "highpassData.empty()" << std::endl;
        return;
    }

}


void SettingsBook::updateBandpassPlotArea ()
{
    if ( !bandpassPlot )
    {
        std::cout << "!bandpassPlot" << std::endl;
        return;
    }

    double bl =  ( double ) fftBandPassBandLoSlider->GetValue() * 0.5 / 22050;
    double sl = ( double ) fftBandPassSlopeLoSlider->GetValue() * 0.5 / 22050;
    double bh =  ( double ) fftBandPassBandHiSlider->GetValue() * 0.5 / 22050;
    double sh = ( double ) fftBandPassSlopeHiSlider->GetValue() * 0.5 / 22050;

    bandpassData.clear();

    double increment = 0.5 / ( double ) fxPlotSize.GetWidth();
    for ( double d = 0.0; d < 0.5; d += increment )
    {
        bandpassData.push_back (
            fftBandpassFilter (
                d, bl, sl,
                bh, sh) );
    }

    bandpassPlot->set ( &bandpassData );

    if ( bandpassData.empty() )
    {
        std::cout << "bandpassData.empty()" << std::endl;
        return;
    }

}


void SettingsBook::updateBandstopPlotArea ()
{
    if ( !bandstopPlot )
    {
        std::cout << "!bandstopPlot" << std::endl;
        return;
    }

    double bl =  ( double ) fftBandStopBandLoSlider->GetValue() * 0.5 / 22050;
    double sl = ( double ) fftBandStopSlopeLoSlider->GetValue() * 0.5 / 22050;
    double bh =  ( double ) fftBandStopBandHiSlider->GetValue() * 0.5 / 22050;
    double sh = ( double ) fftBandStopSlopeHiSlider->GetValue() * 0.5 / 22050;

    bandstopData.clear();

    double increment = 0.5 / ( double ) fxPlotSize.GetWidth();
    for ( double d = 0.0; d < 0.5; d += increment )
    {
        bandstopData.push_back (
            fftBandstopFilter (
                d, bl, sl,
                bh, sh) );
    }

    bandstopPlot->set ( &bandstopData );

    if ( bandstopData.empty() )
    {
        std::cout << "bandstopData.empty()" << std::endl;
        return;
    }

}

void SettingsBook::updateLoudnessPlotArea ()
{
    if ( !loudnessPlot )
    {
        std::cout << "!loudnessPlot" << std::endl;
        return;
    }
    double bl =  ( double ) loudnessBandLoSlider->GetValue() * 0.5 / 22050;
    double sl = ( double ) loudnessSlopeLoSlider->GetValue() * 0.5 / 22050;
    double bh =  ( double ) loudnessBandHiSlider->GetValue() * 0.5 / 22050;
    double sh = ( double ) loudnessSlopeHiSlider->GetValue() * 0.5 / 22050;

    loudnessData.clear();

    double increment = 0.5 / ( double ) generalPlotSize.GetWidth();
    for ( double d = 0.0; d < 0.5; d += increment )
    {
        loudnessData.push_back (
            fftBandstopFilter (
                d, bl, sl,
                bh, sh) );
    }

    loudnessPlot->set ( &loudnessData );

    if ( loudnessData.empty() )
    {
        std::cout << "loudnessData.empty()" << std::endl;
        return;
    }

}


void SettingsBook::updateMasterFilterPlotArea()
{
    if ( !masterFilterPlot )
    {
        std::cout << "!masterFilterPlot" << std::endl;
        return;
    }

    double ( *filter ) ( double frequency,
                         double bandLo, double slopeLo,
                         double bandHi, double slopeHi);
    double bl, sl, bh, sh;

    switch ( masterFilterTypesBox->GetSelection() )
    {
    case ( FFT_LOWPASS ):
        filter = fftLowpassFilter;
        bl = sl = 0.0;
        bh = ( double ) masterFilterBandHiSlider->GetValue() * 0.5 / 22050;
        sh = ( double ) masterFilterSlopeHiSlider->GetValue() * 0.5 / 22050;
        break;
    case ( FFT_HIGHPASS ):
        filter = fftHighpassFilter;
        bl = ( double ) masterFilterBandLoSlider->GetValue() * 0.5 / 22050;
        sl = ( double ) masterFilterSlopeLoSlider->GetValue() * 0.5 / 22050;
        bh = sh = 0.0;
        break;
    case ( FFT_BANDPASS ):
        filter = fftBandpassFilter;
        bl = ( double ) masterFilterBandLoSlider->GetValue() * 0.5 / 22050;
        sl = ( double ) masterFilterSlopeLoSlider->GetValue() * 0.5 / 22050;
        bh = ( double ) masterFilterBandHiSlider->GetValue() * 0.5 / 22050;
        sh = ( double ) masterFilterSlopeHiSlider->GetValue() * 0.5 / 22050;
        break;
    case ( FFT_BANDSTOP ):
        filter = fftBandstopFilter;
        bl = ( double ) masterFilterBandLoSlider->GetValue() * 0.5 / 22050;
        sl = ( double ) masterFilterSlopeLoSlider->GetValue() * 0.5 / 22050;
        bh = ( double ) masterFilterBandHiSlider->GetValue() * 0.5 / 22050;
        sh = ( double ) masterFilterSlopeHiSlider->GetValue() * 0.5 / 22050;
        break;
    default:
        filter = fftLowpassFilter;
        bl = sl = 0.0;
        bh = ( double ) masterFilterBandHiSlider->GetValue() * 0.5 / 22050;
        sh = ( double ) masterFilterSlopeHiSlider->GetValue() * 0.5 / 22050;
        break;
    }

    masterFilterData.clear();

    double increment = 0.5 / ( double ) generalPlotSize.GetWidth();
    for ( double d = 0.0; d < 0.5; d += increment )
    {
        masterFilterData.push_back (
            filter (
                d, bl, sl,
                bh, sh) );
    }

    masterFilterPlot->set ( &masterFilterData );

    if ( masterFilterData.empty() )
    {
        std::cout << "masterFilterData.empty()" << std::endl;
        return;
    }

}

void SettingsBook::updateMasterFilterModPlotArea()
{
    if ( !masterFilterModPlot )
    {
        std::cout << "!masterFilterModPlot" << std::endl;
        return;
    }

    double ( *sweep ) ( double frequency, double limit );
    double depth = 0.01 * masterFilterModulationDepthSlider->GetValue();

    switch ( masterFilterModulationTypesBox->GetSelection() )
    {
    case ( FFT_MOD_SWEEP_IN ):
        sweep = sweepIn;
        break;
    case ( FFT_MOD_SWEEP_OUT ):
        sweep = sweepOut;
        break;
    default:
        sweep = sweepIn;
        break;
    }

    masterFilterModData.clear();

    double increment = 0.5 / ( double ) generalPlotSize.GetWidth();
    for ( double d = 0.0; d < 0.5; d += increment )
    {
        masterFilterModData.push_back (
            ( 1.0 - depth ) + depth * ( sweep ( d, 0.5 ) ) );
    }

    masterFilterModPlot->set ( &masterFilterModData );

    if ( masterFilterModData.empty() )
    {
        std::cout << "masterFilterModData.empty()" << std::endl;
        return;
    }

}


void SettingsBook::updateMasterFilterLfoPlotArea()
{
    if ( !masterFilterLfoPlot )
    {
        std::cout << "!masterFilterLfoPlot" << std::endl;
        return;
    }

    int lfoType = masterFilterLfoTypesBox->GetSelection();
    double masterFilterLfoDepth = 0.01 * ( double ) ( masterFilterLfoDepthSlider->GetValue() );
    double masterFilterLfoPps = ( double ) ( masterFilterLfoPeriodsPerSliceSlider->GetValue() );
    double ( *lfo ) ( double );
    switch ( lfoType )
    {
    case FFT_LFO_SINPLUS:
        lfo = sinPlus;
        break;
    case FFT_LFO_SINPOW2:
        lfo = sinPow2;
        break;
    case FFT_LFO_SAWTOOTH:
        lfo = aaSawTooth;
        break;
    case FFT_LFO_SQUARE:
        lfo = aaSquare;
        break;
    case FFT_LFO_TRIANGLE:
        lfo = aaTriangle;
        break;
    case FFT_LFO_SAMPLEANDHOLD:
        lfo = sampleAndHold;
        break;
    default:
        lfo = passThrough;
        break;
    }

    masterFilterLfoData.clear();

    double increment = ( 2.0 * PI ) / generalPlotSize.GetWidth();
    for ( double d = 0.0; d < 2.0 * PI; d += increment )
    {
        masterFilterLfoData.push_back (
            ( 1.0 - masterFilterLfoDepth ) + masterFilterLfoDepth * lfo ( masterFilterLfoPps * d ));
    }

    masterFilterLfoPlot->set ( &masterFilterLfoData );

    if ( masterFilterLfoData.empty() )
    {
        std::cout << "masterFilterLfoData.empty()" << std::endl;
        return;
    }

}
